## Script to compute radial amplitude from resolution

def amplitude_from_resolution(freq, amplitudes, probe_resolution):
    from emmer.include.emmer_utils import probe_value_from_x
    xdata = freq
    ydata = amplitudes
    if probe_resolution <= 0:
        raise UserWarning("Enter probe resolution > 0A")
    x = 1/probe_resolution
    y = probe_value_from_x(x, xdata, ydata)
    probe_amplitude = y

    return probe_amplitude


## Script to compute FSC at a given resolution for two maps


def compute_fsc_resolution(input_map_1, input_map_2, threshold, apix):
    '''
    Function to calculate the FSC curve from two given input
    input_map_1 : either numpy.ndarray or path to mrc file
    input_map_2 : either numpy.ndarray or path to mrc file
    resolution : float
        resolution at which FSC is to be calculated
    '''
    from emmer.ndimage.fsc import calculate_fsc_curve
    from emmer.include.emmer_utils import probe_value_from_y
    from emmer.ndimage.radial_profile.frequency_array import frequency_array

    fsc_curve = calculate_fsc_curve(input_map_1, input_map_2)
    fsc_curve = np.array(fsc_curve)
    freq = frequency_array(fsc_curve, apix)

    fsc_resolution = probe_value_from_y(y=threshold, xdata=freq, ydata=fsc_curve)

    return fsc_resolution
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_filter(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        


    def test_low_pass_filter(self):
        from emmer.ndimage.filter.low_pass_filter import low_pass_filter
        from emmer.ndimage.map_utils import load_map
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.profile_tools import get_index_at_resolution
        
        sample_emmap_path = self.emmap_path
        
        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            
            filter_cutoff = np.random.uniform(5,20,1)
            output_map_path = os.path.join(tmpdir,'emd5778_map_filtered.mrc')
            emmap, apix = load_map(copied_sample_emmap_path)
            filtered_emmap = low_pass_filter(emmap, apix=apix, cutoff=filter_cutoff)
            
            
            ## test that the filtered map amplitude smaller than the original map

            rp_emmap = compute_radial_profile(emmap)
            rp_filtered_emmap = compute_radial_profile(filtered_emmap)

            freq = frequency_array(rp_emmap, apix=1.2156)
            filter_index = get_index_at_resolution(freq, filter_cutoff/2)

            rp_emmap_0 = rp_emmap[0]
            rp_filtered_emmap_0 = rp_filtered_emmap[0]

            rp_emmap_filtered_index = rp_emmap[filter_index]
            rp_filtered_emmap_filtered_index = rp_filtered_emmap[filter_index]

            self.assertTrue(rp_emmap_filtered_index > rp_filtered_emmap_filtered_index)
            self.assertAlmostEqual(rp_emmap_0, rp_filtered_emmap_0, delta=0.01)
    
    def test_high_pass_filter(self):
        from emmer.ndimage.filter.high_pass_filter import high_pass_filter
        from emmer.ndimage.map_utils import load_map
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.profile_tools import get_index_at_resolution
        
        sample_emmap_path = self.emmap_path
        
        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            
            filter_cutoff = np.random.uniform(5,20,1)
            output_map_path = os.path.join(tmpdir,'emd5778_map_filtered.mrc')
            emmap, apix = load_map(copied_sample_emmap_path)
            filtered_emmap = high_pass_filter(emmap, apix=apix, cutoff=filter_cutoff)
            
            
            ## test that the filtered map amplitude smaller than the original map

            rp_emmap = compute_radial_profile(emmap)
            rp_filtered_emmap = compute_radial_profile(filtered_emmap)

            freq = frequency_array(rp_emmap, apix=1.2156)
            filter_index = get_index_at_resolution(freq, filter_cutoff/2)

            rp_emmap_0 = rp_emmap[0]
            rp_filtered_emmap_0 = rp_filtered_emmap[0]

            rp_emmap_filtered_index = rp_emmap[filter_index]
            rp_filtered_emmap_filtered_index = rp_filtered_emmap[filter_index]

            self.assertTrue(rp_emmap_0 > rp_filtered_emmap_0)
            self.assertAlmostEqual(rp_emmap_filtered_index, rp_filtered_emmap_filtered_index, delta=0.01)
    
    def test_get_cosine_mask(self):
        from emmer.ndimage.filter.smoothen_mask import smoothen_mask
        import random
        from scipy.ndimage import binary_dilation
        ## get sphere at a random location in a map
        zeros = np.zeros((100,100,100))

        center = (random.randint(30,70),random.randint(30,70),random.randint(30,70))

        zeros[center[0],center[1],center[2]] = 1
        dilated_mask = binary_dilation(zeros, iterations=1, structure=np.ones((10,10,10)))

        ## get cosine mask
        cosine_mask = smoothen_mask(dilated_mask, cosine_falloff_length=3)

        ## test that the cosine mask is the same size as the original mask
        self.assertEqual(cosine_mask.shape, zeros.shape)
        unique_vals_cosine = np.unique(cosine_mask.flatten())
        self.assertTrue(len(unique_vals_cosine) > 2)

if __name__ == '__main__':
    unittest.main()
       

       











            




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_bfactor_tools(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        


    def test_estimate_bfactor_map(self):
        ## shake pdb within mask at 2A resolution
        from emmer.ndimage.bfactors.estimate_bfactor_map import estimate_bfactor_map
        from emmer.ndimage.map_utils import load_map
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)

            bfactor, fit = estimate_bfactor_map(copied_sample_emmap_path, wilson_cutoff=10, fsc_cutoff=3.4, return_fit=True)

            self.assertAlmostEqual(bfactor, -215, delta=10)
            self.assertGreater(fit.r_squared(), 0.99)
                
           
    def test_estimate_local_bfactor_map(self):
        from emmer.ndimage.bfactors.estimate_local_bfactor import estimate_local_bfactor
        from emmer.ndimage.map_utils import load_map
        
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)

            bfactor, fit = estimate_local_bfactor(copied_sample_emmap_path, center=(180,179,120), wilson_cutoff=10, fsc_cutoff=3.4, boxsize=22, standard_notation=True, return_fit_quality=True)
            
            self.assertAlmostEqual(bfactor, 95, delta=5)
            self.assertAlmostEqual(fit, 0.89, delta=0.01)
    
    
    def test_bfactor_distribution(self):
        from emmer.ndimage.bfactors.estimate_bfactor_distribution import estimate_bfactor_distribution
        from emmer.ndimage.map_utils import load_map
        
        sample_emmap_path = self.emmap_path
        
        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            
            emmap,_ = load_map(copied_sample_emmap_path)
            binarised_mask = (emmap >= 0.01).astype(int)

            bfactor_distribution = estimate_bfactor_distribution(copied_sample_emmap_path, binarised_mask, wilson_cutoff=10,fsc_cutoff=3.4, boxsize=22, standard_notation=True)

            bfactors = np.array([x[0] for x in bfactor_distribution.values()])
            qfit = np.array([x[1] for x in bfactor_distribution.values()])

            self.assertAlmostEqual(bfactors.mean(), 178 , delta=10)
            self.assertAlmostEqual(qfit.mean(), 0.95 , delta=0.01)






# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_dssp(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        
    def test_secondary_structure_distribution(self):
        from emmer.pdb.pdb_utils import replace_pdb_column_with_arrays, detect_pdb_input
        from emmer.pdb.SSE.get_secondary_structure_residues_content import get_secondary_structure_residues_content

        pdb_path = self.pdb_path
        pdbid="3j5p"

        secondary_structures,secondary_structure_distribution = get_secondary_structure_residues_content(pdbid, pdb_path)

        self.assertAlmostEqual(secondary_structure_distribution['helix'], 0.62, delta=1e-2)
        self.assertAlmostEqual(secondary_structure_distribution['sheet'], 0.03, delta=1e-2)
        self.assertAlmostEqual(secondary_structure_distribution['loop'], 0.16, delta=1e-2)
        self.assertTrue(secondary_structure_distribution['total']==1008)

    def test_split_gemmi_model_based_on_dssp(self):
        from emmer.pdb.pdb_utils import detect_pdb_input
        from emmer.pdb.SSE.split_gemmi_model_based_on_dssp import split_gemmi_model_based_on_dssp

        pdb_path = self.pdb_path
        pdbid="3j5p"

        helix_model, sheet_model, RESIDUE_SKIPPED_STAT = split_gemmi_model_based_on_dssp(pdbid, pdb_path)

        self.assertAlmostEqual(helix_model.count_atom_sites(), 5068, delta=3)
        self.assertAlmostEqual(sheet_model.count_atom_sites(), 227, delta=3)
        self.assertTrue(RESIDUE_SKIPPED_STAT[1]==1008)
        

      


      


## Script to merge two profiles

def merge_two_profiles(amplitudes_low_freq,amplitudes_high_freq,freq, smooth=1, d_cutoff=None, f_cutoff=None):
    '''
    Function to merge two profiles at a cutoff threshold based on differential weighting of two profiles

    Parameters
    ----------
    profile_1 : numpy.ndarray
        
    profile_2 : numpy.ndarray
        same size of profile_1
    freq : numpy.ndarray
        Frequencies corresponding to both profile_1 and profile_2
    d_cutoff : float
        Cutoff frequency defined in terms of distance (unit = A)
    f_cutoff : float
        Cutoff frequency given in terms of spatial frequency (unit = 1/A)
    smooth : float, optional
        smoothening parameter to control the transition region of two profiles

    Returns
    -------
    merged_profile : tuple of two numpy.ndarray
    

    '''
    import numpy as np
    if not (len(freq) == len(amplitudes_low_freq) and len(freq) == len(amplitudes_high_freq)):
        print("Size of two profiles not equivalent. Please check the dimensions and give another input")
        return None
    
    k = smooth
    d = 1 / freq
    
    if d_cutoff is not None:
        d_cutoff = d_cutoff
    
    elif f_cutoff is not None:
        d_cutoff = 1 / f_cutoff
    
    else:
        print("Please enter a cutoff frequency either in terms of spatial frequency (1/A) or distance (A)")
        return None
    
    weight_1 = 1 / (1 + np.exp(k * (d_cutoff - d)))
    weight_2 = 1 - weight_1
    
    merged_profile = weight_1 * amplitudes_low_freq + weight_2 * amplitudes_high_freq
    
    return merged_profile


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 20 13:32:59 2022

@author: alok
"""
import numpy as np

def convert_polar_to_cartesian(polar_vector, multiple=False):
    '''
    Convert polar to cartesian.. Blindly following the formula mentioned here: 
        https://math.libretexts.org/Bookshelves/Calculus/Book%3A_Calculus_(OpenStax)/12%3A_Vectors_in_Space/12.7%3A_Cylindrical_and_Spherical_Coordinates#:~:text=To%20convert%20a%20point%20from,and%20z%3D%CF%81cos%CF%86.
        (accessed: 23-2-2022) 
    
    
    polar_vector: (r, theta, phi) !!
    Theta is the angle in the XY plane

    Parameters
    ----------
    r : float
        
    phi : float 
        first angle in radians
    theta : float
        second angle in radians

    Returns
    -------
    cartesian : numpy.ndarray [1x3]
        (x,y,z)

    '''
    if multiple:
        cartesians = []
        for vector in polar_vector:
            r, theta, phi = vector
            x = r * np.sin(phi) * np.cos(theta)
            y = r * np.sin(phi) * np.sin(theta)
            z = r * np.cos(phi)
            cartesians.append(np.array([x,y,z]))
        return np.array(cartesians)
    else:
        r, theta, phi = polar_vector
        x = r * np.sin(phi) * np.cos(theta)
        y = r * np.sin(phi) * np.sin(theta)
        z = r * np.cos(phi)
    
        cartesian = np.array([x,y,z])
    
        return cartesian



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 20 13:32:59 2022

@author: alok
"""
import numpy as np

def convert_polar_to_cartesian(polar_vector, multiple=False):
    '''
    Convert polar to cartesian.. Blindly following the formula mentioned here: 
        https://math.libretexts.org/Bookshelves/Calculus/Book%3A_Calculus_(OpenStax)/12%3A_Vectors_in_Space/12.7%3A_Cylindrical_and_Spherical_Coordinates#:~:text=To%20convert%20a%20point%20from,and%20z%3D%CF%81cos%CF%86.
        (accessed: 23-2-2022) 
    
    
    polar_vector: (r, theta, phi) !!
    Theta is the angle in the XY plane

    Parameters
    ----------
    r : float
        
    phi : float 
        first angle in radians
    theta : float
        second angle in radians

    Returns
    -------
    cartesian : numpy.ndarray [1x3]
        (x,y,z)

    '''
    if multiple:
        cartesians = []
        for vector in polar_vector:
            r, theta, phi = vector
            x = r * np.sin(phi) * np.cos(theta)
            y = r * np.sin(phi) * np.sin(theta)
            z = r * np.cos(phi)
            cartesians.append(np.array([x,y,z]))
        return np.array(cartesians)
    else:
        r, theta, phi = polar_vector
        x = r * np.sin(phi) * np.cos(theta)
        y = r * np.sin(phi) * np.sin(theta)
        z = r * np.cos(phi)
    
        cartesian = np.array([x,y,z])
    
        return cartesian

def convert_cartesian_to_polar(cartesian):
    '''
    Same as above

    Parameters
    ----------
    x : TYPE
        DESCRIPTION.
    y : TYPE
        DESCRIPTION.
    z : TYPE
        DESCRIPTION.

    Returns
    -------
    Polar : numpy.ndarray

    '''
    x, y, z = cartesian
    r = np.sqrt(np.power(x,2)+np.power(y,2)+np.power(z,2))
    theta = np.arctan(y/x)
    phi = np.arccos(z / r)
    
    polar = np.array([r, theta, phi])
    
    return polar

def get_random_polar_vector(rmsd_magnitude, randomisation="uniform", mean=None):
    if randomisation == "normal":
        if mean is not None:
            r = abs(np.random.normal(loc=mean, scale=rmsd_magnitude))  ## r will be a normally distributed, positive definite variable
            theta = np.random.uniform(0, 2*np.pi)
            phi = np.random.uniform(0, np.pi)
        else:
            r = abs(np.random.normal(loc=0, scale=rmsd_magnitude))  ## r will be a normally distributed, positive definite variable
            theta = np.random.uniform(0, 2*np.pi)
            phi = np.random.uniform(0, np.pi)
                                        
    elif randomisation == "uniform":
        r = np.random.uniform(low=0, high=rmsd_magnitude*2)
        theta = np.random.uniform(0, 2*np.pi)
        phi = np.random.uniform(0, np.pi)
    else:
        raise ValueError("The variable randomisation has only two inputs: normal or uniform")
    
    return np.array([r, theta, phi])

def convert_pdb_to_mrc_position(pdb_position, apix):
    '''
    Convert the real units of positions into indices for the emmap. 
    Note: returns in (Z,Y,X) format
    

    Parameters
    ----------
    pdb_position : list
        list of xyz positions (Angstorm)
    apix : float
        Pixel size 

    Returns
    -------
    mrc_position : list
        List of ZYX positions (index positions)

    '''
    mrc_position = []
    
    for pos in pdb_position:
        [x,y,z] = pos
        int_x, int_y, int_z = int(round(x/apix)), int(round(y/apix)), int(round(z/apix))
        mrc_position.append([int_z, int_y, int_x])
        
    return mrc_position

def convert_mrc_to_pdb_position(mrc_position_list, apix):
    '''
    Convert the real units of positions into indices for the emmap. 
    Note: returns in (Z,Y,X) format
    
    Parameters
    ----------
    mrc_position_list : list
        list of xyz positions (Angstorm)
    apix : float
        Pixel size 

    Returns
    -------
    pdb_position_list : list
        List of XYZ positions (index positions)

    '''
    pdb_position_list = []
    
    for pos in mrc_position_list:
        [nz,ny,nx] = pos
        z, y, x  = nz*apix, ny*apix, nx*apix
        pdb_position_list.append([x, y, z])
        
    return pdb_position_list



def get_spherical_mask(input_emmap, radius_pixels):
    from emmer.ndimage.map_utils import parse_input

    emmap = parse_input(input_emmap)
    n = emmap.shape[0]
    z,y,x = np.ogrid[-n//2:n//2,-n//2:n//2,-n//2:n//2]
    mask = (x**2+y**2+z**2 <= radius_pixels**2).astype(np.int)
    return mask


## Script to perform high pass filtering of cryo EM maps

def high_pass_filter(im, cutoff, apix):
    """
    Returns a high-pass filter image from a tanh filter.
    """
    from emmer.ndimage.filter.filter_utils import calculate_fourier_frequencies, tanh_filter
    import numpy as np

    im_freq     = calculate_fourier_frequencies(im, apix=apix)
    im_filter   = 1-tanh_filter(im_freq, cutoff);
    im_fft      = np.fft.rfftn(im)
    im_fft_filtered = im_fft * im_filter
    im_filtered = np.fft.irfftn(im_fft_filtered)
    return im_filtered

    



# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_pdb_utils(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        
    def test_replace_pdb_column_with_array(self):
        from emmer.pdb.pdb_utils import replace_pdb_column_with_arrays, detect_pdb_input
        
        pdb_path = self.pdb_path
        ## create temporary directory
        from tempfile import TemporaryDirectory
        with TemporaryDirectory() as tmpdir:
            copied_pdb_path = self.copy_files(pdb_path, tmpdir)
            st = detect_pdb_input(copied_pdb_path)
            num_atoms = st[0].count_atom_sites()
            random_array = np.random.randint(0,999,num_atoms)
            pdb_replaced_bfactor = replace_pdb_column_with_arrays(input_pdb=st, replace_column="bfactor", replace_array=random_array)
            pdb_replaced_occupancy = replace_pdb_column_with_arrays(input_pdb=st, replace_column="occ", replace_array=random_array)

            replaced_bfactor_list = []
            for cra_obj in pdb_replaced_bfactor[0].all():
                atom = cra_obj.atom
                replaced_bfactor_list.append(atom.b_iso)
            
            replaced_bfactor_list = np.array(replaced_bfactor_list)

            replaced_occupancy_list = []
            for cra_obj in pdb_replaced_occupancy[0].all():
                atom = cra_obj.atom
                replaced_occupancy_list.append(atom.occ)
            
            replaced_occupancy_list = np.array(replaced_occupancy_list)

            self.assertTrue(np.allclose(replaced_bfactor_list, random_array))
            self.assertTrue(np.allclose(replaced_occupancy_list, random_array))

    def test_set_all_atomic_positions(self):
        from emmer.pdb.pdb_utils import set_all_atomic_positions, get_all_atomic_positions, detect_pdb_input

        pdb_path = self.pdb_path
        ## create temporary directory
        from tempfile import TemporaryDirectory
        with TemporaryDirectory() as tmpdir:
            copied_pdb_path = self.copy_files(pdb_path, tmpdir)
            st = detect_pdb_input(copied_pdb_path)
            num_atoms = st[0].count_atom_sites()
            random_positions = {}
            for i in range(num_atoms):
                random_positions[i] = np.random.randint(0,999, size=3)
            
            random_positions_array = np.array(list(random_positions.values()))
            pdb_changed_positions = set_all_atomic_positions(input_pdb=st, positions_dictionary=random_positions)

            pdb_test_changed_positions = get_all_atomic_positions(input_pdb=pdb_changed_positions, as_dictionary=False) 

            self.assertTrue(np.allclose(random_positions_array, pdb_test_changed_positions))
    
    def test_gemmi_neighbor_analysis(self):
        from emmer.pdb.pdb_utils import find_number_of_neighbors, get_atomic_bfactor_window, detect_pdb_input, get_all_atomic_positions, get_bfactors
        import random

        pdb_path = self.pdb_path
        ## create temporary directory
        from tempfile import TemporaryDirectory
        with TemporaryDirectory() as tmpdir:
            copied_pdb_path = self.copy_files(pdb_path, tmpdir)
            st = detect_pdb_input(copied_pdb_path)

            pdb_positions = get_all_atomic_positions(input_pdb=st, as_dictionary=False)
            pdb_bfactors = get_bfactors(input_pdb=st)

            random_position = random.choice(list(pdb_positions))

            num_neighbors = find_number_of_neighbors(input_pdb=st, atomic_position=random_position, window_size_A=5.0)
            bfactor_window = get_atomic_bfactor_window(input_pdb=st, atomic_position=random_position, window_size_A=5.0)

            self.assertTrue(num_neighbors > 0 and num_neighbors < len(pdb_positions))
            self.assertTrue(bfactor_window > min(pdb_bfactors) and bfactor_window < max(pdb_bfactors))




            



                
           
            


      


from emmer.include.confidenceMapUtil import FDRutil
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np
import argparse, os, sys
import subprocess
import math
import gc
import os.path
from time import sleep


#Author: Maximilian Beckers, EMBL Heidelberg, Sachse Group (2017)


#------------------------------------------------------------------------------------------------------
def localFiltration(map, locResMap, apix, localVariance, windowSize, boxCoord, ECDF):

	#**************************************************
	#**** function to perform a local filtration ******
	#****** according to the local resolution *********
	#**************************************************

	#some initialization
	mapSize = map.shape;
	numX = mapSize[0];
	numY = mapSize[1];
	numZ = mapSize[2];
	
	mean = np.zeros((numX, numY, numZ));
	var = np.zeros((numX, numY, numZ));
	ECDFmap = np.ones((numX, numY, numZ));
	filteredMapData = np.zeros((numX, numY, numZ));

	#transform to numpy array
	locResMapData = np.copy(locResMap);

	#set all resoltuon lower than 2.1 to 2.1
	#locResMapData[locResMapData > 2.5] = 2.5;
	
	locResMapData[locResMapData == 0.0] = 100.0;
	locResMapData[locResMapData >= 100.0] = 100.0;

	#transform to abosulte frequency units(see http://sparx-em.org/sparxwiki/absolute_frequency_units)
	locResMapData = np.divide(apix, locResMapData);
	
	#round to 3 decimals
	locResMapData = np.around(locResMapData, 3);	

	#set resolution search range, 3 decimals exact
	locResArray = np.arange(0, 0.5+0.001 , 0.001);
	
	#set maximum resolution, important as ResMap is masking
	limRes = np.min(locResMapData);
	counter = 0;
	numRes = len(locResArray);	

	#get initial noise statistics
	initMapData = np.copy(map);
	initMean, initVar, _ = FDRutil.estimateNoiseFromMap(initMapData, windowSize, boxCoord);
	noiseMapData = np.random.normal(initMean, math.sqrt(initVar), (100, 100, 100));

	#do FFT of the respective map
	mapFFT = np.fft.rfftn(map);

	#get frequency map
	frequencyMap = FDRutil.calculate_frequency_map(map);

	# Initial call to print 0% progress
	#printProgressBar(counter, numRes, prefix = 'Progress:', suffix = 'Complete', bar_length = 50)
	print("Start local filtering. This might take a few minutes ...");

	counterRes = 0;
	for tmpRes in locResArray:   
		counterRes = counterRes + 1;
		progress = counterRes/float(numRes);
		if counterRes%(int(numRes/20.0)) == 0:
			output = "%.1f" %(progress*100) + "% finished ..." ;
			print(output);
		
		#get indices of voxels with the current resolution	
		indices = np.where(locResMapData == tmpRes);
	
		if (indices[0].size == 0):
			#this resolution is obviously not in the map, so skip
			counter = counter + 1;
			continue;
		elif math.fabs(tmpRes - limRes) < 0.0000001:
			xInd, yInd, zInd = indices[0], indices[1], indices[2];
			
			#do local filtration
			tmpFilteredMapData = FDRutil.lowPassFilter(mapFFT, frequencyMap, tmpRes, map.shape);

			#set the filtered voxels
			filteredMapData[xInd, yInd, zInd] = tmpFilteredMapData[xInd, yInd, zInd];

		else:
			xInd, yInd, zInd = indices[0], indices[1], indices[2];
			#do local filtration
			tmpFilteredMapData = FDRutil.lowPassFilter(mapFFT, frequencyMap, tmpRes, map.shape);
			#set the filtered voxels
			filteredMapData[xInd, yInd, zInd] = tmpFilteredMapData[xInd, yInd, zInd];
			if localVariance == True:
				#estimate and set noise statistic

				if ECDF == 1:
					#if ecdf shall be used, use if to p-vals
					tmpECDF, sampleSort = FDRutil.estimateECDFFromMap(tmpFilteredMapData, windowSize, boxCoord);
					vecECDF = np.interp(tmpFilteredMapData[xInd, yInd, zInd], sampleSort, tmpECDF, left=0.0, right=1.0);
					ECDFmap[xInd, yInd, zInd] = vecECDF; 
				else:
					ECDFmap = 0;

				tmpMean, tmpVar, _ = FDRutil.estimateNoiseFromMap(tmpFilteredMapData, windowSize, boxCoord);
				mean[xInd, yInd, zInd] = tmpMean;
				var[xInd, yInd, zInd] = tmpVar;

	print("Local filtering finished ...");

	return filteredMapData, mean, var, ECDFmap;

#--------------------------------------------------------------------------------------------------
def makeDiagnosticPlot(map, windowSize, locscale, boxCoord):

	#*************************************************************
	#*** function to make diagnostic plot of noise estimation ****
	#*************************************************************

	from matplotlib.backends.backend_pdf import PdfPages
	import matplotlib.gridspec as gridspec

	mapData = map;

	sizeMap = mapData.shape;
	sizePatch = np.array([windowSize, windowSize, windowSize]);
	center = np.array([0.5*sizeMap[0], 0.5*sizeMap[1], 0.5*sizeMap[2]]);
	visMap = np.copy(mapData);
	noiseLabel = (np.mean(visMap)) + 5*np.var(visMap);
	noiseLabel = np.max(visMap.flatten())
	
	#if coordinates are provided, do singleBox estimation
	if boxCoord != 0 | locscale:
		singleBox = True;
	else:
		singleBox = False;

	if locscale & (boxCoord != 0):
		locscale = False; #do not use locscale default
		
	if singleBox == False:
		visMap[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
			int(0.02*sizeMap[1]):(int(0.02*sizeMap[1]) + sizePatch[1]),
			(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))] = noiseLabel;

		visMap[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
			int(0.98*sizeMap[1] - sizePatch[1]):(int(0.98*sizeMap[1] )),
			(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))] = noiseLabel;

		visMap[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
			(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
			int(0.02*sizeMap[2]):(int(0.02*sizeMap[2]) + sizePatch[2])] = noiseLabel;

		visMap[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
			(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
			int(0.98*sizeMap[2] - sizePatch[2]):(int(0.98*sizeMap[2]))] = noiseLabel;

	else:
		if not locscale:
			visMap[int(boxCoord[0]-0.5*sizePatch[0]):(int(boxCoord[0]-0.5*sizePatch[0]) + sizePatch[0]),
				int(boxCoord[1]-0.5*sizePatch[1]):int((boxCoord[1]-0.5*sizePatch[1]) + sizePatch[1]),
				(int(boxCoord[2]-0.5*sizePatch[2])):(int((boxCoord[2]-0.5*sizePatch[2]) + sizePatch[2]))] = noiseLabel;
		else:	
			visMap[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
				int(0.02*sizeMap[1]):(int(0.02*sizeMap[1]) + sizePatch[1]),
				(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))] = noiseLabel;
	
	if boxCoord == 0:
		sliceMapYZ = visMap[int(sizeMap[0]/2.0), :, :];
		sliceMapXZ = visMap[:, int(sizeMap[1]/2.0), :];
		sliceMapXY = visMap[:, :, int(sizeMap[2]/2.0)];
	else:
		sliceMapYZ = visMap[boxCoord[0], :, :];
		sliceMapXZ = visMap[:, boxCoord[1], :];
		sliceMapXY = visMap[:, :, boxCoord[2]];

	#make diagnostics plot	
	plt.gray(); #make grayscale images
	plt.rc('xtick', labelsize=8);    # fontsize of the tick labels
	plt.rc('ytick', labelsize=8);    # fontsize of the tick labels
	gs = gridspec.GridSpec(1, 3);

	#add image of y-z slice
	ax1 = plt.subplot(gs[2]);
	ax1.set_title('Y-Z slice');
	ax1.set_xlabel('Z');
	ax1.set_ylabel('Y');
	ax1.imshow(sliceMapYZ);
	
	#add image of x-z slice
	ax2 = plt.subplot(gs[1]);
	ax2.set_title('X-Z slice');
	ax2.set_xlabel('Z');
	ax2.set_ylabel('X');
	ax2.imshow(sliceMapXZ);

	#add image of x-y slice
	ax3 = plt.subplot(gs[0]);
	ax3.set_title('X-Y slice');
	ax3.set_xlabel('Y');
	ax3.set_ylabel('X');
	ax3.imshow(sliceMapXY);
		
	return plt;
#------------------------------------------------------------------------------------------------
def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, bar_length = 100):
     
    #******************************************
    #** progress bar for local visualization **
    #******************************************	
    """
    Call in a loop to create terminal progress bar
    params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    str_format = "{0:." + str(decimals) + "f}";
    percents = str_format.format(100 * (iteration / float(total)));
    filled_length = int(round(bar_length * iteration / float(total)));
    bar = '#' * filled_length + '-' * (bar_length - filled_length);

    sys.stdout.write('\r%s |%s| %s%s %s' % (prefix, bar, percents, '%', suffix));

    if iteration == total:
        sys.stdout.write('\n');
    sys.stdout.flush();

    return;
#---------------------------------------------------------------------------------
def makeCircularMask(map, sphereRadius):

	#some initialization
	mapSize = map.shape;

	x = np.linspace(-math.floor(mapSize[0]/2.0), -math.floor(mapSize[0]/2.0) + mapSize[0], mapSize[0]);
	y = np.linspace(-math.floor(mapSize[1]/2.0), -math.floor(mapSize[1]/2.0) + mapSize[1], mapSize[1]);
	z = np.linspace(-math.floor(mapSize[2]/2.0), -math.floor(mapSize[2]/2.0) + mapSize[2], mapSize[2]);

	xx, yy, zz = np.meshgrid(x, y, z, indexing='ij');

	mask = np.sqrt(xx**2 + yy**2 + zz**2);

	mask[mask>sphereRadius] = 0.0;

	mask[mask>0.0] = 1.0;

	return mask;
	





import numpy as np
import mrcfile

def load_map(map_path, return_apix = True):
    from emmer.ndimage.map_utils import average_voxel_size
    
    emmap = mrcfile.open(map_path).data
    apix = average_voxel_size(mrcfile.open(map_path).voxel_size)
    
    if return_apix:
        return emmap, apix
    else:
        return emmap

def parse_input(input_map):
    '''
    Function to detect type of input and return a emmap numpy array

    Parameters
    ----------
    input_map : str or numpy array or Mrc object
    string type input should be path/to/emmap.mrc    

    Returns
    -------
    emmap : numpy.ndarray

    '''
    import os
    
    if isinstance(input_map, np.ndarray):
        if len(input_map.shape) == 3:
            return input_map
        else:
            print("You have not input a 3-D numpy array, which cannot be a EM-map")
            return None
    elif isinstance(input_map, str):
        if os.path.exists(input_map):
            emmap = mrcfile.open(input_map).data
            return emmap
        else:
            print("You have not entered a proper path, or the requested file does not exist!")
            return None
    
    
def read_gemmi_map(map_path, return_grid=False):
    '''
    Function to read a map file and return a numpy.ndarray using gemmi.read_ccp4() map

    Parameters
    ----------
    map_path : str
        path/to/emmap.mrc

    Returns
    -------
    emmap : numpy.ndarray
        
    '''
    import gemmi

    gemmi_ccp4Map = gemmi.read_ccp4_map(map_path)
    emmap = np.array(gemmi_ccp4Map.grid, copy=False)
    
    if return_grid:
        return emmap, gemmi_ccp4Map.grid
    else:
        return emmap
    
    
def save_as_mrc(map_data, output_filename, apix=None,origin=None, verbose=False, header=None):
    '''
    Function to save a numpy array containing volume, as a MRC file with proper header

    Parameters
    ----------
    map_data : numpy.ndarray
        Volume data showing the intensities of the EM Map at different points

    apix : float or any iterable
        In case voxelsize in x,y,z are all equal you can also just pass one parameter. 
    output_filename : str
        Path to save the MRC file. Example: 'path/to/map.mrc'
    origin: float or any iterable, optional
        In case origin index in x,y,z are all equal you can also just pass one parameter. 

    Returns
    -------
    Saves MRC .

    '''
    import mrcfile

    with mrcfile.new(output_filename,overwrite=True) as mrc:
        mrc.set_data(np.float32(map_data))
        
        if header is not None:
            mrc.set_extended_header(header)
        
        else:
            if apix is not None:
                #apix_list = [apix['x'], apix['y'], apix['z']]
                ## apix can be either a float or a list. If it's a single number, then the function convert_to_tuple will use it three times
                apix_tuple = convert_to_tuple(apix, num_dims=3)
                rec_array_apix = np.rec.array(apix_tuple, dtype=[('x','<f4'),('y','<f4'),('z','<f4')])
                mrc.voxel_size = rec_array_apix
            else:
                print("Please pass a voxelsize value either as a float or an iterable")
                return 0
            
            if origin is not None:    
                origin_tuple = convert_to_tuple(origin,num_dims=3)
            else:
                origin_tuple = convert_to_tuple(input_variable=0,num_dims=3)
            rec_array_origin = np.rec.array(origin_tuple, dtype=[('x','<f4'),('y','<f4'),('z','<f4')])
            mrc.header.origin = origin_tuple
            
        if verbose:
            print("Saving as MRC file format with following properties: ")
            print("Voxel size", mrc.voxel_size)
            print("Origin", mrc.header.origin)
        
    mrc.close()


def compare_gemmi_grids(grid_1, grid_2, verbose=False):
    '''
    Function to test similarity of two gemmi grids. Test include: 
        (a) Axis order
        (b) Voxelsize
        (c) UnitCell
        (d) Shape
        

    Parameters
    ----------
    grid_1 : gemmi.FloatGrid
        
    grid_2 : gemmi.FloatGrid

    Returns
    -------
    report : pandas.DataFrame
    

    '''
    import pandas as pd
    report = pd.DataFrame()
    report['axis_order'] = [grid_1.axis_order.name,grid_2.axis_order.name]
    report['spacing'] = [grid_1.spacing,grid_2.spacing]
    report['unitcell'] = [grid_1.unit_cell,grid_2.unit_cell]
    report['shape'] = [grid_1.shape, grid_2.shape]
    
    report = report.T
    report['final'] = report[0] == report[1] 
    if verbose:
        if report['final'].all():
            print("The two input grids are same")
        else:
            print("Two input grids are not the same")
            print(report['final'])
    return report
    
def extract_window(im, center, size):
    '''
    Extract a square window at a given location. 
    The center position of the window should be provided.

    Parameters
    ----------
    im : numpy.ndarray
        3D numpy array
    center : tuple, or list, or numpy.array (size=3)
        Position of the center of the window
    size : int, even
        Total window size (edge to edge) as an even number
        (In future could be modified to include different sized window 
        in different directions)
        

    Returns
    -------
    window : numpy.ndarray
        3D numpy array of shape (size x size x size)

    '''
    z,y,x = center
    window = im[z-size//2:z+size//2, y-size//2:y+size//2, x-size//2:x+size//2]
    return window

def binarizeMap(emmap, threshold):
	binarised = (emmap>=threshold).astype(np.int_)
	return binarised

def average_voxel_size(voxel_size_record):
    apix_x = voxel_size_record.x
    apix_y = voxel_size_record.y
    apix_z = voxel_size_record.z
    
    average_apix = (apix_x+apix_y+apix_z)/3
    
    return average_apix

   
def get_sphere(radius):
    '''
    Function to return a window, with a spherical mask. Size of the window defined by the radius.
    Parameters
    ----------
    radius : int
        Radius of sphere, in pixels

    Returns
    -------
    sphere : numpy.ndarray
        Shape: (rad+1) x (rad+1)

    '''
    z,y,x = np.ogrid[-radius: radius+1, -radius: radius+1, -radius: radius+1]
    sphere = (x**2+y**2+z**2 <= radius**2).astype(int)
    return sphere


def moving_average(array,window=5):
    return np.convolve(array,np.ones(window), 'same')/window

def normalise(x):
    if isinstance(x,np.ndarray):
        return x/x.max()
    else:
        x = np.array(x)
        return x/x.max()

def dilate_mask(mask, radius, iterations=1):
    '''
    Dilate mask with spherical structures

    Parameters
    ----------
    mask : numpy.ndarray
        Skeleton structure of a set of atoms
    radius : int
        Cutoff radius used for binary dilation
    iterations : int, optional
        Number of iterations for binary dilation The default is 1.

    Returns
    -------
    dilated : numpy.ndarray
        Dilated mask

    '''
    from scipy.ndimage import binary_dilation
    from emmer.ndimage.map_utils import get_sphere
    
    dilated = binary_dilation(mask, structure=get_sphere(radius), iterations=iterations).astype(int)
        
    return dilated

def convert_to_tuple(input_variable, num_dims=3):
    '''
    Convert any variable, or iterable into a tuple. If a scalar is input then a tuple is generated with same variable
    based on number of dimensions mentioned in num_dims

    Parameters
    ----------
    input_variable : any
        scalar, or any iterable
    num_dims : int, optional
        Length of tuple. The default is 3.
        
    Returns
    -------
    output_tuple : tuple

    '''
    
    if hasattr(input_variable, '__iter__'):
        if len(input_variable) == num_dims:
            output_tuple = tuple(input_variable)
            return output_tuple
        else:
            print("Input variable dimension {} doesn't match expected output dimension {}".format(len(input_variable), num_dims))
    else:
        output_list = [input_variable for temporary_index in range(num_dims)]
        output_tuple = tuple(output_list)
        return output_tuple
    
def get_all_voxels_inside_mask(binarised_mask, mask_threshold=1):
    all_inside_mask = np.asarray(np.where(binarised_mask>=mask_threshold)).T.tolist()
    return all_inside_mask





    
    
    
    
################################### CODE HELL ######################################

# =============================================================================
# def convert_to_tuple_2(input_variable, num_dims=3):
#     '''
#     Convert any variable, or iterable into a tuple. If a scalar is input then a tuple is generated with same variable
#     based on number of dimensions mentioned in num_dims
# 
#     Parameters
#     ----------
#     input_variable : any
#         scalar, or any iterable
#     num_dims : int, optional
#         Length of tuple. The default is 3.
#         
#     Returns
#     -------
#     output_tuple : tuple
# 
#     '''
#     
#     if hasattr(input_variable, '__iter__'):
#         if isinstance(input_variable, np.recarray):
#             input_variable = [input_variable['x'], input_variable['y'], input_variable['z']]
#         
#         if len(input_variable) == num_dims:
#             output_tuple = tuple(input_variable)
#             return output_tuple
#         else:
#             print("Input variable dimension {} doesn't match expected output dimension {}".format(len(input_variable), num_dims))
#     else:
#         output_list = [input_variable for temporary_index in range(num_dims)]
#         output_tuple = tuple(output_list)
#         return output_tuple  
#     
#     
# =============================================================================



## Script to perform band pass filtering of cryo EM maps

from emmer.ndimage.filter.low_pass_filter import low_pass_filter
from emmer.ndimage.filter.high_pass_filter import high_pass_filter

def band_pass_filter(im, minimum_resolution, maximum_resolution, apix):
    
    im_low_pass = low_pass_filter(im, maximum_resolution, apix)
    im_high_pass = high_pass_filter(im_low_pass, minimum_resolution, apix)
    
    return im_high_pass
    


## Script to compute B factor from a radial profile

def estimate_bfactor_standard(freq, amplitude, wilson_cutoff, fsc_cutoff, return_amplitude=False, return_fit_quality=False, standard_notation=False):
    '''
    From a given radial profile, estimate the b_factor from the high frequency cutoff

    Parameters
    ----------
    freq : numpy.ndarray
        Frequency array
    amplitude : numpy.ndarray
        Amplitudes
    wilson_cutoff : float
        Frequency from which wilson statistics are valid. Units: Angstorm
    fsc_cutoff : float
        FSC resolution calculated at 0.143 (for halfmaps). Units: Angstorm
        

    Returns
    -------
    b_factor : float
        The estimated b factor
    
    amp : float
        The estimated amplitude of the exponential fit

    '''
    from scipy.optimize import curve_fit
    from sklearn.metrics import r2_score
    from emmer.ndimage.radial_profile.profile_tools import crop_profile_between_resolution
    import numpy as np
    def linear_fit(xdata,slope,const):
        ydata = const + slope*xdata
        return ydata
    
    cropped_freq, cropped_amplitude = crop_profile_between_resolution(freq, amplitude, wilson_cutoff, fsc_cutoff)
    
    xdata = cropped_freq**2
    ydata = np.log(cropped_amplitude)
        
    param, _ = curve_fit(linear_fit,xdata,ydata)
    
    if standard_notation:
        b_factor = -1 * param[0] * 4   ## Inverse of slope
    else:
        b_factor = param[0] * 4
    
    exp_fit_amplitude = np.exp(param[1])
    
    
    y_pred = linear_fit(xdata, slope=param[0], const=param[1])
    r2 = r2_score(y_true=ydata, y_pred=y_pred)
    
    if return_amplitude:
        if return_fit_quality:
            return b_factor,exp_fit_amplitude, r2
        else:
            return b_factor,exp_fit_amplitude
    else:
        if return_fit_quality:
            return b_factor, r2
        else:
            return b_factor


# -*- coding: utf-8 -*-
"""
Created on Sun Jul 18 14:52:11 2021
"""

# get_dssp contains several functions to analyse secondary structure
# information in PDB structures using Biopython

# global imports
from Bio.PDB import PDBParser
from Bio.PDB.DSSP import DSSP
from Bio.PDB import PDBList
from emmer.pdb.pdb_utils import detect_pdb_input
#%% functions
  
def link_gemmi_model_with_dssp(gemmi_model,dssp):
     '''
     gemmi_model: gemmi.Model() 
     dssp: dictionary which is the output of Bio.PDB.DSSP.DSSP() function
     
     return:
     linked_dictionary[(chain_name,res_id)] = relevant_dssp_key
     '''
     linked_dictionary = {}
     for key in dssp.keys():
          chain_name = key[0]
          res_id = key[1][1]
          linked_dictionary[(chain_name,res_id)] = key
          
     return linked_dictionary



## Function to return frequency array from a array of amplitudes and pixel size
import numpy as np
def frequency_array(amplitudes=None, apix=None, n=None):
    '''
    Function to return frequency array from a array of amplitudes and pixel size
    
    Parameters
    ----------
    amplitudes : ndarray
        Array of amplitudes
    apix : float
        Pixel size in Angstroms
    n : int
        Number of points in the frequency array
    Returns
    -------
    frequency_array : ndarray
        Array of frequencies
    '''
    ## Check inputs
    if amplitudes is None and n is None:
        raise ValueError("Must provide either amplitudes or n")
    if apix is None:
        raise ValueError("Must provide apix")
    if amplitudes is not None and n is not None:
        raise ValueError("Must provide either amplitudes or n")

    if amplitudes is not None:
        n = len(amplitudes)
    else:
        n = int(n)
    ## Calculate frequency array
    freq = np.linspace(1/(apix*n),1/(apix*2),n,endpoint=True)

    return freq

    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 15 00:40:10 2021

@author: alok
"""
import numpy as np

       
## Script to add new functions to the ndimage branch

def measure_mask_parameters(mask_path=None, mask=None,apix=None,mask_threshold=0.99,protein_density=1.35,average_atomic_weight=13.14,verbose=True,detailed_report=False):
    
    from emmer.ndimage.map_utils import load_map
    from scipy.constants import Avogadro
    '''
    Function to calculated parameters of a EM Mask map

    Parameters
    ----------
    mask_path : string 
        Path to mask file
    edge_threshold : float 
        The threshold to strictly binarize the FDR map at the edges
    protein_density : float, optional
        Average protein density to calculate number of atoms. The default is 1.35.
    average_atomic_weight : float, optional
        Atomic weight of an "average atom present in protein". 
        Found using 54% carbon, 20% oxygen and 16% nitrogen. The default is 12.066.
    verbose : bool, optional
        Print statistics if True. The default is True.

    Returns
    -------
    num_atoms : int
        Estimated number of atoms based on mask volume, protein density and average atomic weight
    

    '''
    if mask_path is not None:
        mask, apix = load_map(mask_path)
    
    assert mask is not None
    assert apix is not None
    
    
    ang_to_cm = 1e-8
    
    binarise_mask = (mask>=mask_threshold).astype(np.int_)
    
    mask_vol = binarise_mask.sum()*(apix*ang_to_cm)**3
    mask_vol_A3 = binarise_mask.sum()*apix**3
    
    protein_mass = protein_density * mask_vol
    num_moles = protein_mass / average_atomic_weight
    num_atoms = int((num_moles * Avogadro).round())
    maskshape = mask.shape
    mask_dims = [maskshape[0]*apix,maskshape[1]*apix,maskshape[2]*apix]
    
    
    if verbose:
        print("Mask parameters calculated are: \n"+
              "Mask sum voxels: "+str(round(mask.sum(),3))+"\n"+
              "Mask volume: "+str(round(mask_vol_A3,3))+" A^3 \n"+
              "Protein mass: "+str(round(1e21*protein_mass))+" zg\n"+
              "Num atoms: "+str(num_atoms)+"\n") 
        
    if not detailed_report:
        return num_atoms,mask_dims
    else:
        return mask_vol_A3, protein_mass, num_atoms, mask_dims,maskshape
    

def resample_map(emmap, emmap_size_new=None, apix=None, apix_new=None, order=1):
    '''
    Function to resample an emmap in real space using linear interpolation 

    Parameters
    ----------
    emmap : numpy.ndimage
        
    emmap_size_new : tuple 
        
    apix : float
        
    apix_new : float
        

    Returns
    -------
    resampled_emmap

    '''
    from scipy.ndimage import zoom
    if emmap_size_new is None:
        if apix is not None and apix_new is not None:
            resample_factor = apix/apix_new
        else:
            raise UserWarning("Provide either (1) current pixel size and new pixel size or (2) new emmap size")
    
    else:
        try:
            resample_factor = emmap_size_new[0] / emmap.shape[0]
        except:
            raise UserWarning("Please provide proper input: emmap_size_new must be a tuple")
    
    resampled_image = zoom(emmap, resample_factor, order=order)
    
    return resampled_image

      
def resample_image(im, imsize_new=None, apix=1.0, apix_new=None):
    """Returns a real image or volume resampled by cropping/padding its Fourier Transform
    """
    import numpy as np
    
    imsize = np.array(im.shape)
    if np.any(imsize_new == None) and apix_new == None:
        return im
    elif apix_new != None:
        imsize_new = np.round(imsize * apix / apix_new).astype('int')
        pad_factor = imsize_new/imsize
    elif imsize_new != None:
        imsize_new = np.array(imsize_new)
        pad_factor = imsize_new/imsize

    ft = np.fft.fftn(im)
    ft = np.fft.fftshift(ft)
    ft = pad_or_crop_image(ft, pad_factor)
    ft = np.fft.ifftshift(ft)
    return np.fft.ifftn(ft).real  

def pad_or_crop_image(im, pad_factor=None, pad_value = None, crop_image=False):
    """Returns the original image cropped or padded by pad_factor and pad_value; pad_factor being a fraction/multiple of original image size.
       Default behaviour is zero padding.
    """
    if np.any(pad_factor == None):
        return im
    else:
        pad_factor = np.round(np.multiply(pad_factor,np.array(im.shape))).astype('int')

        if pad_value == None:
            pad_value = 0

        if len(im.shape) == 2:       
            if (pad_factor[0] <= im.shape[0] or pad_factor[1] <= im.shape[1]):
                crop_image = True    
            
            if crop_image:
                crop_im = im[im.shape[0]//2-pad_factor[0]//2:im.shape[0]//2+pad_factor[0]//2+pad_factor[0]%2, :]
                crop_im = crop_im[:, im.shape[1]//2-pad_factor[1]//2:im.shape[1]//2+pad_factor[1]//2+pad_factor[1]%2]
                return crop_im
            else:
                pad_im = np.pad(im, ((pad_factor[0]//2-im.shape[0]//2, pad_factor[0]//2-im.shape[0]//2+pad_factor[0]%2), (0,0)), 'constant', constant_values=(pad_value,))
                pad_im = np.pad(pad_im, ((0,0),(pad_factor[1]//2-im.shape[1]//2, pad_factor[1]//2-im.shape[1]//2+pad_factor[1]%2 )), 'constant', constant_values=(pad_value,))
                return pad_im         
            
        elif len(im.shape) == 3:
            if (pad_factor[0] <= im.shape[0] or pad_factor[1] <= im.shape[1] or pad_factor[2] <= im.shape[2]):
                crop_image = True

            if crop_image:
                crop_im = im[im.shape[0]//2-pad_factor[0]//2:im.shape[0]//2+pad_factor[0]//2+pad_factor[0]%2, :, :]
                crop_im = crop_im[:, im.shape[1]//2-pad_factor[1]//2:im.shape[1]//2+pad_factor[1]//2+pad_factor[1]%2, :]
                crop_im = crop_im[:, :, im.shape[2]//2-pad_factor[2]//2:im.shape[2]//2+pad_factor[2]//2+pad_factor[2]%2]
                return crop_im

            else:
                pad_im = np.pad(im, ((pad_factor[0]//2-im.shape[0]//2, pad_factor[0]//2-im.shape[0]//2+pad_factor[0]%2), (0,0), (0,0) ), 'constant', constant_values=(pad_value,))
                pad_im = np.pad(pad_im, ((0,0), (pad_factor[1]//2-im.shape[1]//2, pad_factor[1]//2-im.shape[1]//2+pad_factor[1]%2 ), (0,0)), 'constant', constant_values=(pad_value,))
                pad_im = np.pad(pad_im, ((0,0), (0,0), (pad_factor[2]//2-im.shape[2]//2, pad_factor[2]//2-im.shape[2]//2+pad_factor[2]%2)), 'constant', constant_values=(pad_value,))
                return pad_im


## Script to compute radial profile of a given volume

def compute_radial_profile(vol, return_indices=False):
    '''
    Computes the radial profile of a given volume

    Parameters
    ----------
    vol : numpy.ndarray
        Input array
    center : list, optional
        DESCRIPTION. The default is [0,0,0].
    return_indices : bool, optional
        

    Returns
    -------
    radial_profile : numpy.ndarray (1D)
        Radial profile
        

    '''
    import numpy as np

    dim = vol.shape
    m = np.mod(vol.shape,2)

    ps = np.abs( np.fft.rfftn(vol) )
    if not return_indices:
        x, y, z = np.indices(ps.shape)
        radii = np.sqrt(x**2 + y**2 + z**2)
        radii = radii.astype(int)
    else:
        [x, y, z] = np.mgrid[-dim[0]//2+m[0]:(dim[0]-1)//2+1, -dim[1]//2+m[1]:(dim[1]-1)//2+1, 0:dim[2]//2+1]
        x = np.fft.ifftshift(x)
        y = np.fft.ifftshift(y)
        radii = np.sqrt(x**2 + y**2 + z**2)
        radii = radii.astype(int)
    
    radial_profile = np.bincount(radii.ravel(), ps.ravel()) / np.bincount(radii.ravel())
    # exclude corner frequencies
    radial_profile = radial_profile[0:int(ps.shape[0]/2)]
    if not return_indices:
        return radial_profile
    else:
        return radial_profile, radii


# -*- coding: utf-8 -*-
"""
Created on Sun Jul 18 14:52:11 2021
"""

# get_dssp contains several functions to analyse secondary structure
# information in PDB structures using Biopython

# global imports
from Bio.PDB import PDBParser
from Bio.PDB.DSSP import DSSP
from Bio.PDB import PDBList
from emmer.pdb.pdb_utils import detect_pdb_input
#%% functions

def get_dssp(pdbid,pdb_path):  
    parser = PDBParser()
    structure = parser.get_structure(pdbid,pdb_path)
    model = structure[0]
    try:
        dssp = DSSP(model,pdb_path)
        print("Successfully performed DSSP! \n")
    except Exception as e:
        print("Problem with DSSP")
        print(e)
        raise
    
    return dssp



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec  3 14:11:29 2020

@author: Alok, Arjen, Maarten, Stefan, Reinier 
"""

# pdb_tools contains several useful fucntions for common manipulations with
# pdb structures, making use of the gemmi package. functions are classified 
# into pdb_tools when they can be considered an application on their own
# but do not have so many distinct features that they warrent their own script.

# global imports
import mrcfile
import gemmi
import numpy as np

#%% functions

def shake_pdb_within_mask(pdb_path, mask_path, rmsd_magnitude,mask_threshold=0.5, verbose=False):
    from emmer.pdb.pdb_utils import detect_pdb_input
    from emmer.ndimage.map_utils import load_map
    from emmer.pdb.pdb_utils import get_all_atomic_positions, set_all_atomic_positions
    from emmer.pdb.convert.convert_polar_to_cartesian import convert_polar_to_cartesian
    from emmer.pdb.convert.convert_mrc_to_pdb_position import convert_mrc_to_pdb_position
    from emmer.pdb.convert.convert_pdb_to_mrc_position import convert_pdb_to_mrc_position
    from emmer.pdb.pdb_tools.compute_rmsd_two_pdb import compute_rmsd_two_pdb
    from emmer.ndimage.map_utils import get_all_voxels_inside_mask
    from tqdm import tqdm
    import random   
    
    ## Inputs
    st = detect_pdb_input(pdb_path)
    
    mask,apix = load_map(mask_path)
    
    outside_mask = np.logical_not(mask>=mask_threshold)
    
    voxel_positions_mask = get_all_voxels_inside_mask(mask, mask_threshold=0.5)
    pdb_positions_mask = np.array(convert_mrc_to_pdb_position(voxel_positions_mask, apix))
    
       
    ## Get all atomic positions, in real units (XYZ)
    atomic_positions_values = get_all_atomic_positions(st)  ## XYZ
    # Get shake vector     
    if verbose:
        print("Shaking the input structure with input RMSD of {}...".format(rmsd_magnitude))
    shake_radii = np.random.uniform(0, rmsd_magnitude*2, size=len(atomic_positions_values))
    shake_thetas = np.random.uniform(0, 2*np.pi, size=len(atomic_positions_values))
    shake_phis = np.random.uniform(0, np.pi, size=len(atomic_positions_values))
    
    shake_vectors_polar = np.column_stack((shake_radii, shake_thetas, shake_phis))
    if verbose:
        print("Checking for atoms outside the mask...")
    
    shaken_atomic_position_native = list(atomic_positions_values + convert_polar_to_cartesian(shake_vectors_polar, multiple=True))
    rmsd_native = compute_rmsd_two_pdb(shaken_atomic_position_native, atomic_positions_values, use_gemmi_structure=False)
    
    shaken_atomic_position= shaken_atomic_position_native.copy()
    
    shaken_mrc_position_list = [tuple(x) for x in convert_pdb_to_mrc_position(shaken_atomic_position, apix)]  ## ZYX
       
    
    
    
    num_atoms_outside = 0
    
    from sklearn.neighbors import KDTree
    tree = KDTree(pdb_positions_mask)
    
    for i,mrc_pos in enumerate(tqdm(shaken_mrc_position_list, "Validating positions")):
        if outside_mask[mrc_pos[0],mrc_pos[1],mrc_pos[2]]:
            neighborhood_indices_list = tree.query_radius(shaken_atomic_position[i:i+1], r=rmsd_magnitude*2) 
            random_index = random.choice(list(neighborhood_indices_list)[0])
            random_position = pdb_positions_mask[random_index] + np.random.uniform(0,apix/2,3)
                
            shaken_atomic_position[i] = random_position
            num_atoms_outside += 1

    if verbose:
        print("{} atoms found outside the mask! Randomly placed them inside the mask within its sphere of influence.".format(num_atoms_outside))
        print("Done... Now converting into PDB")
    
    shaken_atomic_positions_dictionary = {}
    for i,atomic_position in enumerate(shaken_atomic_position):
        shaken_atomic_positions_dictionary[i] = atomic_position
    
    shaken_structure = set_all_atomic_positions(st, shaken_atomic_positions_dictionary)


    if verbose:    
        rmsd = compute_rmsd_two_pdb(st, shaken_structure)
        print("RMSD between input structure and native shaken structure: {} A".format(round(rmsd_native,2)))
        print("RMSD between the input and output structure is: {} A".format(round(rmsd,2)))
        
    return shaken_structure      



## Script to compute the center of mass of a given input emmap.

def get_center_of_mass(emmap, apix, background_threshold=None):
    '''
    Computes the center of mass of a given input emmap. 
    Note: converts the negative intensities to positive to calculate COM

    Parameters
    ----------
    emmap : numpy.ndarray
        
    apix : float or any iterable
        Voxelsize

    Returns
    -------
    com_real : numpy.ndarray
        units: (A * A * A)

    '''
    from scipy.ndimage import center_of_mass
    from emmer.pdb.convert.convert_mrc_to_pdb_position import convert_mrc_to_pdb_position

    if background_threshold is not None:
        emmap[emmap < background_threshold] = 0
    
    COM_mrc_position = list(center_of_mass(abs(emmap)))
    COM_pdb_position = convert_mrc_to_pdb_position([COM_mrc_position], apix)[0]
    
    
    return COM_pdb_position
    


import numpy as np
import subprocess
import math
import gc
import os
import sys


#Author: Maximilian Beckers, EMBL Heidelberg, Sachse Group

#-------------------------------------------------------------------------------------
def estimateNoiseFromMap(map, windowSize, boxCoord):

	#**************************************************
	#****** function to estimate var an mean from *****
	#**** nonoverlapping boxes outside the particle ***
	#**************************************************

	if boxCoord == 0:
		#extract a sample of pure noise from the map
		sizeMap = map.shape;
		sizePatch = np.array([windowSize, windowSize, windowSize]);
		center = np.array([0.5*sizeMap[0], 0.5*sizeMap[1], 0.5*sizeMap[2]]);
		
		sampleMap1 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(0.02*sizeMap[1]):(int(0.02*sizeMap[1]) + sizePatch[1]),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];

		sampleMap2 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(0.98*sizeMap[1] - sizePatch[1]):(int(0.98*sizeMap[1])),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];
	
		sampleMap3 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
		int(0.02*sizeMap[2]):(int(0.02*sizeMap[2]) + sizePatch[2])];

		sampleMap4 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
		int(0.98*sizeMap[2]) - sizePatch[2]:(int(0.98*sizeMap[2]))];

		#concatenate the two samples
		sampleMap = np.concatenate((sampleMap1, sampleMap2, sampleMap3, sampleMap4), axis=0);

	else:
		sizePatch = np.array([windowSize, windowSize, windowSize]);
		center = np.array(boxCoord);
		sampleMap = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(center[1]-0.5*sizePatch[1]):(int(center[1]-0.5*sizePatch[1]) + sizePatch[1]),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];	
	
	#estimate variance and mean from the sample
	mean = np.mean(sampleMap);
	var = np.var(sampleMap);

	if var == 0.0:
		print("Variance is estimated to be 0. You are probably estimating noise in a masked region. Exit ...")
		sys.exit();

	return mean, var, sampleMap;
#-------------------------------------------------------------------------------------
def estimateNoiseFromMapInsideMask(map, mask):

	#**************************************************
	#****** function to estimate var an mean from *****
	#******* map outside the user provided mask *******
	#**************************************************

	mask[mask<=0.5] = 0.0;
	mask[mask>0.0] = 1000.0;
	mask[mask<1000.0] = 1.0;
	mask[mask==1000.0] = 0.0;

	sampleMap = np.copy(map)*mask;
	sampleMap = sampleMap[sampleMap != 0.0];
	
	#estimate variance and mean from the sample
	mean = np.mean(sampleMap);
	var = np.var(sampleMap);
	
	
	return mean, var, sampleMap;

#-------------------------------------------------------------------------------------
def estimateECDFFromMap(map, windowSize, boxCoord):

	#**************************************************
	#****** function to estimate empirical cumul. *****
	#**** distribution function from solvent area *****
	#**************************************************

	if boxCoord == 0:
		#extract a sample of pure noise from the map
		sizeMap = map.shape;
		sizePatch = np.array([windowSize, windowSize, windowSize]);
		center = np.array([0.5*sizeMap[0], 0.5*sizeMap[1], 0.5*sizeMap[2]]);
		sampleMap1 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(0.02*sizeMap[1]):(int(0.02*sizeMap[1]) + sizePatch[1]),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];

		sampleMap2 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(0.98*sizeMap[1] - sizePatch[1]):(int(0.98*sizeMap[1])),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];
	
		sampleMap3 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
		int(0.02*sizeMap[2]):(int(0.02*sizeMap[2]) + sizePatch[2])];

		sampleMap4 = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		(int(center[1]-0.5*sizePatch[1])):(int((center[1]-0.5*sizePatch[1]) + sizePatch[1])), 
		int(0.98*sizeMap[2]) - sizePatch[2]:(int(0.98*sizeMap[2]))];

		#conatenate the two samples
		sampleMap = np.concatenate((sampleMap1, sampleMap2, sampleMap3, sampleMap4), axis=0);
	
	elif boxCoord == -1:
		sampleMap = map;
	
	else:
		sizePatch = np.array([windowSize, windowSize, windowSize]);
		center = np.array(boxCoord);
		sampleMap = map[int(center[0]-0.5*sizePatch[0]):(int(center[0]-0.5*sizePatch[0]) + sizePatch[0]),
		int(center[1]-0.5*sizePatch[1]):(int(center[1]-0.5*sizePatch[1]) + sizePatch[1]),
		(int(center[2]-0.5*sizePatch[2])):(int((center[2]-0.5*sizePatch[2]) + sizePatch[2]))];	
	
	#estimate ECDF from map
	sampleMap = sampleMap.flatten();	
	
	#downsize the sample
	finalSampleSize = min(100000, sampleMap.size);
	sampleMap = np.random.choice(sampleMap, finalSampleSize, replace = False);
	numSamples = sampleMap.size;
	sampleMapSort = np.sort(sampleMap);

	minX = sampleMapSort[0];
	maxX = sampleMapSort[numSamples-1];
	numInterval = numSamples;
	spacingX = (maxX - minX)/(float(numInterval));

	ECDF = np.zeros(numInterval);	
	for index in range(numInterval):
		val = sampleMapSort[index];
		ECDF[index] = ((sampleMapSort[sampleMapSort<= val]).size)/float(numSamples);
	
	return ECDF, sampleMapSort;

#------------------------------------------------------------------------------------
def getCDF(x, ECDF, sampleMapSort):

	#****************************************************
	#********* get the value of the CDF at point x ******
	#******* CDF : Cumulative distribution function *****
	#****************************************************
	
	numSamples = sampleMapSort.size;
	minX = sampleMapSort[0];
	maxX = sampleMapSort[numSamples-1];


	if x >= maxX:
		CDFval = 1.0;
	elif x <= minX:
		CDFval = 0.0;
	else:
		#get the index in the ECDF array		
		index = np.searchsorted(sampleMapSort, x) - 1;	
		CDFval = ECDF[index];

	return CDFval;
#------------------------------------------------------------------------------------
def AndersonDarling(sample):

	#********************************************
	#*** Anderson-Darling test for normality ****
	#********************************************

	sample = np.random.choice(sample, min(10000,sample.size), replace=False);
	sampleMapSort = np.sort(sample);
	numSamples = sampleMapSort.size;

	Ad = -1.0*numSamples;
	for i in range(numSamples):
		CDF_Yi = 0.5 * (1.0 + math.erf(sampleMapSort[i]/math.sqrt(2.0)));
		CDF_Yn = 0.5 * (1.0 + math.erf(sampleMapSort[numSamples-i-1]/math.sqrt(2.0)));
		if CDF_Yi == 0:
			CDF_Yi = 0.000001;
		if CDF_Yi == 1:
			CDF_Yi = 0.999999;
		if CDF_Yn == 0:
			CDF_Yn = 0.000001;
		if CDF_Yn == 1:
			CDF_Yn = 0.999999;

		#calculate the Anderson-Darling test statistic
		Ad = Ad - (1.0/float(numSamples)) * (2*(i+1)-1)*(math.log(CDF_Yi) + (math.log(1.0-CDF_Yn)));

	#do adjustment for estimation of mean and variance, as unknown before
	Ad = Ad*(1 + 0.75/float(numSamples) + 2.25/float(numSamples*numSamples));

	#calculate p-values
	# R.B. D'Augostino and M.A. Stephens, Eds., 1986, Goodness-of-Fit Techniques, Marcel Dekker
	try:
		if Ad >= 0.6:
			pVal = math.exp(1.2937 - 5.709*(Ad) + 0.0186*Ad*Ad);
		elif 0.34<Ad<0.6:
			pVal = math.exp(0.9177 - 4.279*Ad - 1.38 * Ad*Ad);
		elif 0.2 < Ad <= 0.34:
			pVal = 1 - math.exp(-8.318 + 42.796*Ad - 59.938*Ad*Ad);
		else:
			pVal = 1 - math.exp(-13.436 + 101.14 * Ad - 223.73 * Ad*Ad);
	except:
		pVal = -1.0;

	return Ad, pVal, numSamples;


#------------------------------------------------------------------------------------
def KolmogorowSmirnow(ECDF, sampleMapSort):

	#***********************************************
	#***** KS test by supremum of distance *********
	#*********** between CDF and ECDF **************
	#***********************************************

	#some initialization
	numSamples = sampleMapSort.size;
	X = np.linspace(-5, 5, 200000);
	vectorizedErf = np.vectorize(math.erf);

	#maximum distances between CDF and ECDF over the whole defintion region
	Y_stdNorm = 0.5 * (1.0 + vectorizedErf(X/math.sqrt(2.0)));
	Y_ecdf = np.interp(X, sampleMapSort, ECDF, left=0.0, right=1.0);
	Dn = np.amax(np.absolute(np.subtract(Y_stdNorm,Y_ecdf)));

	#get Kolmogorow-Smirnow test statistic
	KS_testStat = math.sqrt(numSamples)*Dn;

	#maximum distances between CDF and ECDF for tail regions
	X_tail_right = X[X>2.0];
	X_tail_left = X[X<-2.0];
	X_tail = np.concatenate((X_tail_right, X_tail_left));
	Y_stdNorm = 0.5 * (1.0 + vectorizedErf(X_tail/math.sqrt(2.0)));
	Y_ecdf = np.interp(X_tail, sampleMapSort, ECDF, left=0.0, right=1.0);
	Dn_tail = np.amax(np.absolute(np.subtract(Y_stdNorm,Y_ecdf)));

	return KS_testStat, Dn, Dn_tail, numSamples;

#-----------------------------------------------------------------------------------
def checkNormality(map, windowSize, boxCoord):

	#***************************************
	#** check normal distribution ass. *****
	#***************************************

	print('Checking the normal distribution assumption ...');

	mean, var, _ = estimateNoiseFromMap(map, windowSize, boxCoord);

	map = np.subtract(map, mean);
	tMap = np.multiply(map, (1.0/(math.sqrt(var))));
	map = np.copy(tMap);
		
	#get maximum distances between ECDF and CDF
	ECDFvals, sampleSort = estimateECDFFromMap(map, windowSize, boxCoord);
	KSstat, Dn, Dn_tail, n = KolmogorowSmirnow(ECDFvals, sampleSort);
	output = "Maximum Distance Dn between ECDF and CDF: Dn=" + " %.4f" %Dn + ", in Tail:" + " %.4f" %Dn_tail + ". Sample size used: " + repr(n);
	print(output);

	#do Anderson-Darling test for normality
	AnDarl, pVal, n = AndersonDarling(sampleSort);
	output = "Anderson-Darling test summary: " + repr(AnDarl) + ". p-Value: " + "%.4f" %pVal + ". Sample size used: " + repr(n);
	if pVal != -1.0:
		print(output);
	else:
		pVal = -1.0;		

	if (Dn_tail > 0.01):
		output = "WARNING: Deviation in the tail areas between the normal distribution and the empircal CDF is higher than 1%. If boxes for background noise estimation are set properly, please consider using the flag -ecdf to use the empirical CDF instead of the normal distribution."
		print(output);

#------------------------------------------------------------------------------------
def studentizeMap(map, mean, var):
	
	#****************************************
	#********* normalize map ****************
	#****************************************
	
	if np.isscalar(var):
		studMap = np.subtract(map, mean);
		studMap = np.multiply(studMap, (1.0/(math.sqrt(var))));
	else: #if local variances are known, use them
		var[var == 0] = 1000;
		studMap = np.subtract(map, mean);
		studMap = np.divide(studMap, np.sqrt(var));
		var[var == 1000] = 0.0;
		studMap[var == 0.0] = 0.0;
		
	return studMap;


#-----------------------------------------------------------------------------------
def calcQMap(map, mean, var, ECDF, windowSize, boxCoord, mask, method, test):

	#*****************************************
	#***** generate qMap of a 3D density *****
	#*****************************************

	#get some map data
	sizeMap = map.shape;

	#calculate the test statistic
	if np.isscalar(var):
		#map[map == 0.0] = -100000000;
		tmap = np.subtract(map, mean);   
		tMap = np.multiply(tmap, (1.0/(math.sqrt(var))));
		map = np.copy(tMap);
	else:
		var[var==0.0] = 1000.0; #just to avoid division by zero
		tmap = np.subtract(map, mean); 	
		tMap = np.divide(tmap, np.sqrt(var));
		var[var==1000.0] = 0.0;
		#upadte the mask, necessary as resmap is masking as well
		mask = np.multiply(mask,var);

	if test == 'rightSided':
		tMap[tMap<0] = -10000000.0;
	elif test == 'leftSided':
		tMap[tMap>0] = 10000000.0;

	#calculate the p-Values
	print('Calculating p-Values ...');
	pMap = np.zeros(sizeMap);
	
	if np.isscalar(ECDF): 
		if ECDF == 0: 
			vectorizedErf = np.vectorize(math.erf);
			erfMap = vectorizedErf(tMap/math.sqrt(2.0));
			#erf2Map = special.erf(tMap/math.sqrt(2.0));

			pMapRight = 1.0 - (0.5*(1.0 + erfMap)); 
			pMapLeft = (0.5*(1.0 + erfMap));
	
		else:
			#if ecdf shall be used, use if to p-vals
			ECDF, sampleSort = estimateECDFFromMap(map, windowSize, boxCoord);
			print('start ECDF calculation ...');	
			vecECDF = np.interp(map, sampleSort, ECDF, left=0.0, right=1.0);
			pMapRight = 1.0 - vecECDF;
			pMapLeft = vecECDF;

			#************************************
			#****** make diagnostic plots *******
			#************************************
			X = np.linspace(-5, 5, 10000);
			vectorizedErf = np.vectorize(math.erf);
			Y1 = np.interp(X, sampleSort, ECDF, left=0.0, right=1.0);
			Y2 = (0.5*(1.0 + vectorizedErf(X/math.sqrt(2.0))));
			np.savetxt('ECDF.txt', Y1, delimiter=';');
			import matplotlib.pyplot as plt
			from matplotlib.backends.backend_pdf import PdfPages
			import matplotlib.gridspec as gridspec
			plt.rc('xtick', labelsize=8);    # fontsize of the tick labels
			plt.rc('ytick', labelsize=8);    # fontsize of the tick labels
			pp = PdfPages('ECDF_vs_CDF.pdf');
			gs = gridspec.GridSpec(1, 2);
			
			ax1 = plt.subplot(gs[0]);
			ax1.set_title('Full CDF');
			ax1.plot(X, Y1);
			ax1.plot(X, Y2);
	    	#do test plots
			X = np.linspace(2, 5, 10000);
			Y1 = np.interp(X, sampleSort, ECDF, left=0.0, right=1.0);
			Y2 = (0.5*(1.0 + vectorizedErf(X/math.sqrt(2.0))));
			ax2 = plt.subplot(gs[1]);
			ax2.set_title('Tail CDF');
			ax2.plot(X, Y1);
			ax2.plot(X, Y2);
			pp.savefig();
			pp.close();
			plt.close();
	else:
		pMapRight = 1 - ECDF;
		pMapLeft = ECDF;

	if test == 'twoSided':
		pMap = np.minimum(pMapLeft, pMapRight);
	elif test == 'rightSided':
		pMap = pMapRight;
		pMap[tMap==-10000000.0] = 1.0
	elif test == 'leftSided':
		pMap = pMapLeft;
		pMap[tMap==10000000.0] = 1.0;
	
	#take the p-values in the mask	
	binaryMask = np.copy(mask);
	binaryMask[ binaryMask != 0.0 ] = 1.0;
	binaryMask[ binaryMask ==  0.0] = np.nan;

	pMap = pMap * binaryMask;
	pFlat = pMap.flatten();
	pInBall = pFlat[~np.isnan(pFlat)];

	#do FDR control, i.e. calculate the qMap
	print('Start FDR control ...');
	pAdjValues = pAdjust(pInBall, method);	

	pAdjFlat = np.copy(pFlat);
	pAdjFlat[~np.isnan(pAdjFlat)] = pAdjValues;
	pAdjMap = np.reshape(pAdjFlat, sizeMap);
	pAdjMap[np.isnan(pAdjMap)] = 1.0;

	return pAdjMap;

#---------------------------------------------------------------------------------
def pAdjust(pValues, method):

	#***********************************
	#***** FDR correction methods ******
	#***********************************

	numPVal = len(pValues);

	print("Sorting p-values ...")
	pSortInd = np.argsort(pValues);
	pSort = pValues[pSortInd];
	
	pAdjust = np.zeros(numPVal);
	prevPVal = 1.0;

	#use expansion for harmonic series
	Hn = math.log(numPVal) + 0.5772 + 0.5/numPVal - 1.0/(12*numPVal**2) + 1.0/(120*numPVal**4);    

	print("Adjusting p-values ...");
	if method =='BH': #do benjamini-hochberg procedure
		for i in range(numPVal-1, -1, -1):
			pAdjust[i] = min(prevPVal, pSort[i]*numPVal/(i+1.0));
			prevPVal = pAdjust[i];	
	
	elif method == 'BY': #do benjamini yekutieli procedure
		for i in range(numPVal-1, -1, -1):
			pAdjust[i] =  min(prevPVal, pSort[i]*(numPVal/(i+1.0))*Hn);
			prevPVal = pAdjust[i];
	elif method == "Holm":
		prevPVal = 0.0;
		for i in range(numPVal):
			tmpPVal = (numPVal - i)*pSort[i];
			pAdjust[i] = max(prevPVal, tmpPVal);
			prevPVal = pAdjust[i];
		pAdjust[pAdjust>1.0] = 1.0;
	elif method == "Hochberg":
		for i in range(numPVal-1, -1, -1):
			pAdjust[i] = min(prevPVal, pSort[i]*(numPVal-i));
			prevPVal = pAdjust[i];		
	else:
		print('Please specify a method. Execution is stopped ...');
		quit();	

	#sort back to the original order
	pSortIndOrig = np.argsort(pSortInd);

	return pAdjust[pSortIndOrig];

#---------------------------------------------------------------------------------
def binarizeMap(map, threshold):

	#***********************************
	#*binarize map at given threshold **
	#***********************************

	binMap = np.array(map);
	binMap[binMap <= threshold] = 0;
	binMap[binMap > threshold] = 1;

	#finally invert the map
	binMap = np.subtract(np.ones(binMap.shape), binMap);
  	
	return binMap;

#---------------------------------------------------------------------------------
def tanh_filter(x, cutoff):

	#**********************************
	#******* tanh lowpass filter ******
	#**********************************

	filter_fall_off = 0.1;

	fx = 1.0 - (1.0 - 0.5*(np.tanh((np.pi*(x+cutoff)/(2*filter_fall_off*cutoff))) - np.tanh((np.pi*(x-cutoff)/(2*filter_fall_off*cutoff)))));

	return fx;
#---------------------------------------------------------------------------------
def calculate_frequency_map(map):

    sizeMap = map.shape;

    #calc frequency for each voxel
    freqi = np.fft.fftfreq(sizeMap[0], 1.0);
    freqj = np.fft.fftfreq(sizeMap[1], 1.0);
    freqk = np.fft.rfftfreq(sizeMap[2], 1.0);

    sizeFFT = np.array([freqi.size, freqj.size, freqk.size]);
    FFT = np.zeros(sizeFFT);

    freqMapi = np.copy(FFT);
    for j in range(sizeFFT[1]):
        for k in range(sizeFFT[2]):
            freqMapi[:,j,k] = freqi*freqi;

    freqMapj = np.copy(FFT);
    for i in range(sizeFFT[0]):
        for k in range(sizeFFT[2]):
            freqMapj[i,:,k] = freqj*freqj;

    freqMapk = np.copy(FFT);
    for i in range(sizeFFT[0]):
        for j in range(sizeFFT[1]):
            freqMapk[i,j,:] = freqk*freqk;
    
    frequencyMap = np.sqrt(freqMapi + freqMapj + freqMapk);

    return frequencyMap;

#---------------------------------------------------------------------------------
def lowPassFilter(mapFFT, frequencyMap, cutoff, shape):

	#**********************************
	#*** filter in fourier domain *****
	#**********************************

	sizeMap = mapFFT.shape;

	#do filtering of the map
	filterMap = tanh_filter(frequencyMap, cutoff);
	filteredftMap = filterMap*mapFFT;
	
	#do iverse FFT
	try: #optional with pyfftw
		import pyfftw
		import multiprocessing

		# get number of cpus
		numCores = multiprocessing.cpu_count();

		fftObject = pyfftw.builders.irfftn(filteredftMap, shape, threads = numCores);
		filteredMap = fftObject();

	except:
		filteredMap = np.fft.irfftn(filteredftMap, shape);


	filteredMap = np.real(filteredMap);

	return filteredMap;

#---------------------------------------------------------------------------------
def printSummary(args, time):

	#***********************************
	#** print a Summary of the job *****
	#***********************************

	print("*** Done ***");
	print(" ");
	print("******** Summary ********");

	output = "Elapsed Time: " + repr(time);
	print(output);

	#print input map filename
	splitFilename = os.path.splitext(os.path.basename(args.em_map));
	output = "Input EM-map: " + splitFilename[0] + ".mrc"; 
	print(output);

	#print model map filename
	if args.model_map is not None:
		splitFilename = os.path.splitext(os.path.basename(args.model_map));
		output = "LocScale was done with the input model-map: " + splitFilename[0] + ".mrc";
		print(output);

		#print window size
		if args.window_size is not None:
			w = int(math.ceil(args.window_size / 2.) * 2);
		else:
			w = int(round(7 * 3 * args.apix));
		output = "Window size: " + repr(w);

	#print local resolution map filename
	if args.locResMap is not None:
		splitFilename = os.path.splitext(os.path.basename(args.locResMap));
		output = "Input local resolution map: " + splitFilename[0] + ".mrc";
		print(output);
	
	#print output filenames
	if args.outputFilename is not None:
		splitFilename = os.path.splitext(os.path.basename(args.outputFilename));
	else:
		splitFilename = os.path.splitext(os.path.basename(args.em_map));
	
	#print specifications for locScale and local filtering
	if args.model_map is not None:
		output = "Output LocScale map: " + splitFilename[0] + "_scaled" + ".mrc";
		print(output);
		output = "Output confidence Map: " + splitFilename[0] + "_confidenceMap" + ".mrc";
		print(output);
		
		if args.stepSize is not None:
			output = "Step size used for the sliding window moves: " + repr(args.stepSize);
		else:
			output = "Step size used for the sliding window moves: " + repr(5);
		print(output);

	else:
		if args.locResMap is not None:
			output = "Output locally filtered map: " + splitFilename[0] + "_locFilt" + ".mrc";
			print(output);
			output = "Output confidence Map: " + splitFilename[0] + "_confidenceMap" + ".mrc";
			print(output);
		else:
			output = "Output confidence Map: " + splitFilename[0] + "_confidenceMap" + ".mrc";
			print(output);

	#print method used for FDR-control
	if args.method is None:
		output = "Multiple Testing was controlled with: BY";			
	else:
		output = "Multiple Testing  was controlled with: " + args.method;
	print(output);





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Oct  1 11:30:49 2021

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_profile_tools_functions(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    
    def test_frequency_array(self):
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        apix_test = np.random.uniform(low=0.5,high=2.5)
        length = np.random.randint(low=100, high=1000)
        
        
        ## Tests based on saved dataset
        
        freq_test = frequency_array(n=length, apix=apix_test)
        
        self.assertAlmostEqual(freq_test[0], 1/(apix_test*length), delta=1e-6)
        self.assertAlmostEqual(freq_test[-1], 1/(apix_test*2), delta=1e-6)
    
    def test_compute_radial_profile(self):
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        
        emmap= mrcfile.open(self.emmap_path).data
        modmap = mrcfile.open(self.modmap_path).data
        
        rp_emmap_test = compute_radial_profile(emmap)
        rp_modmap_test = compute_radial_profile(modmap)
        
        freq = frequency_array(rp_emmap_test, 1.2156)
        
        rp_emmap_target = self.validate['profile_tools_output']['rp_emmap']
        rp_modmap_target = self.validate['profile_tools_output']['rp_modmap']
        
        self.assertTrue((rp_emmap_test==rp_emmap_target).any())
        self.assertTrue((rp_modmap_test==rp_modmap_target).any())
    
    def test_match_bfactors(self):
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        from emmer.ndimage.radial_profile.scale_profiles import scale_profiles

        emmap= mrcfile.open(self.emmap_path).data
        modmap = mrcfile.open(self.modmap_path).data
        
        rp_emmap_test = compute_radial_profile(emmap)
        rp_modmap_test = compute_radial_profile(modmap)
        freq = frequency_array(rp_emmap_test, 1.2156)

        freq_match, scaled_amplitude = scale_profiles((freq, rp_emmap_test), 
                                                    (freq, rp_modmap_test), 
                                                    wilson_cutoff=4.7, fsc_cutoff=3.4)
        
        amp_match_target = self.validate['profile_tools_output']['amp_matched']
        sr=abs(scaled_amplitude/amp_match_target)
        
        self.assertTrue(np.allclose(sr, np.ones(len(sr)), atol=0.2))
        self.assertTrue(np.allclose(freq_match, freq))
       
    
   
    def test_bfactor_and_resample(self):
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.estimate_bfactor_standard import estimate_bfactor_standard
        from emmer.ndimage.radial_profile.estimate_bfactor_pwlf import estimate_bfactor_pwlf
        emmap= mrcfile.open(self.emmap_path).data
        rp_emmap_test = compute_radial_profile(emmap)
        freq = frequency_array(rp_emmap_test, 1.2156)
        bfactor_std_test = estimate_bfactor_standard(freq, rp_emmap_test, wilson_cutoff=4.7, fsc_cutoff=3.4)
        bfactor_pwlf_test, dc_amp, (fit, z, slope) = estimate_bfactor_pwlf(freq, rp_emmap_test, wilson_cutoff=9,fsc_cutoff=3.4, num_segments=3) 
                                                                      
        
        from emmer.ndimage.radial_profile.profile_tools import resample_1d
        resampled_rp_test = resample_1d(freq, rp_emmap_test, num=1000, xlims=[1/9,1/3.4])
        
      
        self.assertTrue((resampled_rp_test[1]==self.validate['profile_tools_output']['resampled_profile'][1]).any())
        self.assertAlmostEqual(bfactor_std_test,self.validate['profile_tools_output']['bfactor_std'],delta=10)
        self.assertAlmostEqual(bfactor_pwlf_test,self.validate['profile_tools_output']['bfactor_pwlf'], delta=10)
    
    def test_crop_profile_between_resolution(self):
        from emmer.ndimage.radial_profile.profile_tools import crop_profile_between_resolution
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile

        emmap= mrcfile.open(self.emmap_path).data
        rp_emmap_test = compute_radial_profile(emmap)
        freq_test = frequency_array(rp_emmap_test, 1.2156)

        cropped_rp_test = crop_profile_between_resolution(freq_test, rp_emmap_test, 9, 3.4)
        cropped_freq, cropped_amp = cropped_rp_test

        self.assertAlmostEqual(cropped_freq[-1], 1/3.4, delta=1e-2)
        self.assertAlmostEqual(cropped_freq[0], 1/9, delta=1e-2)
    
    def test_get_index_at_resolution(self):
        from emmer.ndimage.radial_profile.profile_tools import get_index_at_resolution
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
        emmap= mrcfile.open(self.emmap_path).data
        rp_emmap_test = compute_radial_profile(emmap)
        freq_test = frequency_array(rp_emmap_test, 1.2156)
        index_test = get_index_at_resolution(freq_test, 3.4)
        self.assertEqual(index_test, 90)

    def test_measure_debye(self):
        from emmer.ndimage.radial_profile.measure_debye_effect_magnitude import measure_debye_effect_magnitude
        from emmer.ndimage.radial_profile.frequency_array import frequency_array
        from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile

        emmap= mrcfile.open(self.emmap_path).data
        rp_emmap_test = compute_radial_profile(emmap)
        freq_test = frequency_array(rp_emmap_test, 1.2156)
        debye_magnitude = measure_debye_effect_magnitude(freq_test, rp_emmap_test, wilson_cutoff=10, fsc_cutoff=3.4)
        self.assertAlmostEqual(debye_magnitude,107.3984, delta=2)
    

    


        
    
            
            
        
        
    
        




# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_map_utils(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        


    def test_load_map(self):
        
        from emmer.ndimage.map_utils import load_map
        
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            emmap, apix = load_map(copied_sample_emmap_path)

            self.assertTrue(emmap.shape == (256,256,256))
            self.assertAlmostEqual(apix, 1.2156, delta=0.1)
        
    def test_parse_input(self):
        from emmer.ndimage.map_utils import parse_input
        from emmer.ndimage.map_utils import load_map
        
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            emmap, apix = load_map(copied_sample_emmap_path)

            emmap_from_path = parse_input(copied_sample_emmap_path)
            emmap_from_array = parse_input(emmap)
            emmap_from_nonsense = parse_input("this_is_a_nonsense_path")

            self.assertTrue(np.array_equal(emmap_from_path, emmap))
            self.assertTrue(np.array_equal(emmap_from_array, emmap))
            self.assertTrue(emmap_from_nonsense is None)
    
    def test_read_gemmi_path(self):
        from emmer.ndimage.map_utils import read_gemmi_map
        import gemmi
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            copied_sample_emmap_path = self.copy_files(sample_emmap_path, tmpdir)
            emmap, grid = read_gemmi_map(copied_sample_emmap_path, return_grid=True)

            self.assertTrue(emmap.shape == (256,256,256))
            xyz_axis_order = grid.axis_order.value
                       
            self.assertTrue(xyz_axis_order == 1)
    
    def test_save_as_mrc(self):
        from emmer.ndimage.map_utils import save_as_mrc
        import os
        ## create temporary directory
        from tempfile import TemporaryDirectory
        with TemporaryDirectory() as tmpdir:
            temp_mrc_path = os.path.join(tmpdir,'temp.mrc')
            zeros = np.zeros((256,256,256))
            save_as_mrc(zeros, temp_mrc_path, apix=3)
            
            self.assertTrue(os.path.exists(temp_mrc_path))
    
    def test_compare_grid(self):
        from emmer.ndimage.map_utils import compare_gemmi_grids, save_as_mrc, read_gemmi_map
        sample_emmap_path = self.emmap_path

        ## create temporary directory
        from tempfile import TemporaryDirectory

        with TemporaryDirectory() as tmpdir:
            random_map_1 = np.random.uniform(0,1,(256,256,256))       
            random_map_2 = np.random.uniform(0,1,(256,256,256))
            random_map_3 = np.random.uniform(0,1,(256,256,256))

            temp_mrc_path_1 = os.path.join(tmpdir,'temp_1.mrc')
            temp_mrc_path_2 = os.path.join(tmpdir,'temp_2.mrc')
            temp_mrc_path_3 = os.path.join(tmpdir,'temp_3.mrc')

            save_as_mrc(random_map_1, temp_mrc_path_1, apix=3)
            save_as_mrc(random_map_2, temp_mrc_path_2, apix=3)
            save_as_mrc(random_map_3, temp_mrc_path_3, apix=2)

            emmap_1, grid_1 = read_gemmi_map(temp_mrc_path_1, return_grid=True)
            emmap_2, grid_2 = read_gemmi_map(temp_mrc_path_2, return_grid=True)
            emmap_3, grid_3 = read_gemmi_map(temp_mrc_path_3, return_grid=True)
            

            self.assertTrue(compare_gemmi_grids(grid_1, grid_2)['final'].all())
            self.assertFalse(compare_gemmi_grids(grid_1, grid_3)['final'].all())
    

    
            



## Script to compute bfactor from radial profile using PWLF 

def estimate_bfactor_pwlf(freq,amplitudes,wilson_cutoff,fsc_cutoff, return_all=True, num_segments=3):
    '''
    Function to automatically find out linear region in a given radial profile 


    @Manual{pwlf,
            author = {Jekel, Charles F. and Venter,     Gerhard},
            title = {{pwlf:} A Python Library for Fitting 1D Continuous Piecewise Linear Functions},
            year = {2019},
            url = {https://github.com/cjekel/piecewise_linear_fit_py}
}

    Parameters
    ----------
    freq : numpy.ndarray
        
    amplitudes : numpy.ndarray
        

    Returns
    -------
    start_freq_in_angstorm, estimated_bfactor

    '''
    import numpy as np
    import pwlf
    from emmer.ndimage.radial_profile.profile_tools import crop_profile_between_resolution
    cropped_freq, cropped_amplitude = crop_profile_between_resolution(freq, amplitudes, wilson_cutoff, fsc_cutoff)
    
    x_data = cropped_freq**2
    y_data = np.log(cropped_amplitude)
    
    piecewise_linfit = pwlf.PiecewiseLinFit(x_data, y_data)
    z = piecewise_linfit.fit(n_segments=num_segments)
    
    slopes = piecewise_linfit.calc_slopes()
    
    bfactor = slopes[-1] * 4
    
    amplitude_zero_freq = piecewise_linfit.predict(0)
    
    if return_all:
        return bfactor, amplitude_zero_freq, (piecewise_linfit, z, slopes)
    else:
        return bfactor


## Script to estimate the local bfactor of a cryo EM density map at a given point in the map

def estimate_local_bfactor(emmap_path, center, wilson_cutoff, fsc_cutoff, boxsize=None, standard_notation=True, return_fit_quality=False):
    '''
    Script to estimate the local bfactor of a cryo EM density map at a given point in the map.
    emmap_path: path to the cryo EM density map
    center: the center of the local bfactor estimation
    wilson_cutoff: the Wilson score cutoff for the local bfactor estimation
    fsc_cutoff: the FSC cutoff for the local bfactor estimation
    boxsize: the size of the box to extract around the point in the map
    standard_notation: whether to use the standard notation for the bfactor (True)
    return_fit_quality: whether to return the fit quality of the bfactor (True)

    returns:
    bfactor: the estimated bfactor (float)
    '''

    from emmer.ndimage.radial_profile.estimate_bfactor_standard import estimate_bfactor_standard
    from emmer.ndimage.radial_profile.compute_radial_profile import compute_radial_profile
    from emmer.ndimage.radial_profile.frequency_array import frequency_array
    from emmer.ndimage.map_utils import extract_window, load_map
    from emmer.include.emmer_utils import round_up_to_even

    emmap, apix = load_map(emmap_path)
       
    if boxsize is None:
        boxsize = round_up_to_even(25 / apix)
    else:
        boxsize = round_up_to_even(boxsize)

    emmap_window = extract_window(emmap, center, boxsize)
            
    rp_local = compute_radial_profile(emmap_window)
    freq = frequency_array(rp_local, apix)
            
    bfactor,qfit = estimate_bfactor_standard(freq, rp_local, wilson_cutoff, fsc_cutoff, standard_notation=standard_notation, return_fit_quality=True)
    
    if return_fit_quality:
        return bfactor, qfit
    else:
        return bfactor


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec  3 14:11:29 2020

@author: Alok, Arjen, Maarten, Stefan, Reinier 
"""

# pdb_tools contains several useful fucntions for common manipulations with
# pdb structures, making use of the gemmi package. functions are classified 
# into pdb_tools when they can be considered an application on their own
# but do not have so many distinct features that they warrent their own script.

# global imports
import mrcfile
import gemmi
import numpy as np

#%% functions

def find_wilson_cutoff(input_pdb=None, mask_path=None, mask = None, apix=None, method='Singer', return_as_frequency=False, verbose=False):
    '''
    Function to find the cutoff frequency above which Wilson statistics hold true. If a PDB file is passed either as a gemmi structure as a PDB path, then radius of gyration is found rigorously by the mean distance to center of mass of protein. If a mask is passed, however, then radius of gyration is estimated from the num_atoms calculated from the mask volume. 
    
Reference: 
    1) Estimating Radius of gyration from num_atoms John J. Tanner,  2016 (https://www.ncbi.nlm.nih.gov/pmc/articles/PMC5053138/)
    
    2) Estimating cutoff frequency: Amit Singer, 2021 (https://www.biorxiv.org/content/10.1101/2021.05.14.444177v1.full)
    
    3) Estimating cutoff frequency: Guiner method - Rosenthal & Henderson, 2003 (https://doi.org/10.1016/j.jmb.2003.07.013)

    Parameters
    ----------
    model_path : string, optional
        path to pdb file. The default is None.
    input_gemmi_st : gemmi.Structure(), optional
        
    mask_path : string, optional
        path to mask. The default is None.
    method : string, optional
        Method used to find the cutoff frequency. Two accepted values are: 'Singer', and 'Rosenthal_Henderson' (case insensitive). The default is 'Singer'.
    return_as_frequency : TYPE, optional
        DESCRIPTION. The default is False.

    Returns
    -------
    TYPE
        DESCRIPTION.

    '''
    
    from emmer.ndimage.map_utils import load_map
    from emmer.ndimage.map_tools import measure_mask_parameters
    from emmer.pdb.pdb_utils import find_radius_of_gyration, detect_pdb_input
    
    if input_pdb is not None:
        gemmi_st = detect_pdb_input(input_pdb)
        num_atoms = gemmi_st[0].count_atom_sites()
        Rg = find_radius_of_gyration(input_pdb=gemmi_st)

    else:
        if mask_path is not None:
            mask, apix = load_map(mask_path)
        
        assert mask
        assert apix
    
        mask_vol_A3, protein_mass, num_atoms, mask_dims,maskshape = measure_mask_parameters(mask=mask, apix=apix, detailed_report=True)

        R_constant = 2 #A
        v = 0.4 # Exponent derived empirically Ref. 1 for monomers and oligomers
        Rg = R_constant * num_atoms**v
  
    if verbose:
        print("Number of atoms: {} \nRadius of Gyration: {:.2f}".format(num_atoms,Rg))
    if method.lower() == 'rosenthal_henderson':
        d_cutoff = 2*np.pi*Rg
        f_cutoff = 1/d_cutoff
    elif method.lower() == 'singer':
        ko = num_atoms**(-1/12) # Guiner transition non-dimensional
        
        Ro = Rg * np.cbrt(1/num_atoms) # Unit cell dimension around each atom
        
        f_cutoff = ko/Ro
        d_cutoff = 1/f_cutoff
    else:
        raise UserWarning("Enter rosenthal_henderson or singer as method")
    
    if verbose:
        print("Frequency cutoff: {:.2f} (in 1/A) \n".format(f_cutoff))
        print("Frequency cutoff: {:.2f} (in A) \n ".format(d_cutoff))
    
    if return_as_frequency:
        return f_cutoff
    else:
        return d_cutoff



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 19 13:53:12 2022

@author: alok
"""

import numpy as np


def plot_radial_profile(freq,list_of_profiles, squared_amplitudes=False, legends=None, normalise=False, font=28,showlegend=True, showPoints=True, alpha=0.05, variation=None, yticks=None, logScale=True, ylims=None, xlims=None, crop_freq=None):
    import matplotlib.pyplot as plt
    from matplotlib.pyplot import cm
    from locscale.include.emmer.ndimage.profile_tools import crop_profile_between_resolution
    
    '''
    Given a list of amplitudes, plot them against a common frequency axis.

    Parameters
    ----------
    freq : np.ndarray
        Common frequency axis. Same size as the profiles in list of profiles
    list_of_profiles : list 
        List of amplitude profiles all having same size.
        list_of_profiles = [profile_1(type=np.ndarray), profile_2(type=np.ndarray), ...]
    colors : list, optional
        Custom color list. Max 6 entries. The default is ['r','g','b','k','y','m'].
    legends : lsit of string, optional
        Attach a legend corresponding to each profile in the list of profile. 
    font : int, optional
        fontsize for the plots. The default is 12.
    showlegend : bool, optional
        If you need to hide the legends, set this parameter to False

    Returns
    -------
    None.

    '''
        
    i = 0
    colors = cm.rainbow(np.linspace(0,1,len(list_of_profiles)))
    

    if legends is None:
        legends = ["profile_"+str(i) for i in range(len(list_of_profiles))]
    if len(list_of_profiles) <= 50:
        fig = plt.figure()
        ax1 = fig.add_subplot(111)
        ax1.grid(False)
        ax2 = ax1.twiny()
        #plt.xticks(fontsize=font)
        #plt.yticks(fontsize=font)
        if logScale:
            for profile in list_of_profiles:
                if crop_freq is not None:
                    freq, profile = crop_profile_between_resolution(freq, profile, crop_freq[0], crop_freq[1])
                if showPoints:
                    ax1.plot(freq**2,np.log(profile),c=colors[i], linewidth=1, marker="o")
                else:
                    ax1.plot(freq**2,np.log(profile),c=colors[i], linewidth=1)
                i += 1
            
            ax2.set_xticks(ax1.get_xticks())
            ax2.set_xbound(ax1.get_xbound())
            ax2.set_xticklabels([round(1/np.sqrt(x),1) for x in ax1.get_xticks()])
            if showlegend:
                ax1.legend(legends)
            ax1.set_xlabel(r'Spatial Frequency, $d^{-2} (\AA^{-2})$')
            ax1.set_ylabel(r'$ln  \langle \mid F \mid \rangle $ ')
            ax2.set_xlabel(r'$d (\AA)$')
        else:
            for profile in list_of_profiles:
                if crop_freq is not None:
                    freq, profile = crop_profile_between_resolution(freq, profile, crop_freq[0], crop_freq[1])
                if showPoints:
                    ax1.plot(freq,profile,c=colors[i], linewidth=1, marker="o")
                else:
                    ax1.plot(freq,profile,c=colors[i], linewidth=1)
                i += 1
            
            
            if showlegend:
                ax1.legend(legends)
        
                
            ax2.set_xticks(ax1.get_xticks())
            ax2.set_xbound(ax1.get_xbound())
            ax2.set_xticklabels([round(1/x,1) for x in ax1.get_xticks()])
            
    
            ax1.set_xlabel(r'Spatial Frequency, $1/d [\AA^{-1}]$')
            ax1.set_ylabel(r'Normalised $ \langle F \rangle $')
            ax2.set_xlabel('$d [\AA]$')
            
        
        
    elif variation is None:
        
        profile_list = np.array(list_of_profiles)
        average_profile = np.einsum("ij->j", profile_list) / len(profile_list)
        
        variation = []
        for col_index in range(profile_list.shape[1]):
            col_extract = profile_list[:,col_index]
            variation.append(col_extract.std())

        variation = np.array(variation)
        
        y_max = average_profile + variation
        y_min = average_profile - variation

        fig = plt.figure()
        
        ax1 = fig.add_subplot(111)
        ax1.grid(False)
        ax2 = ax1.twiny()
        
        if logScale:
            if crop_freq is not None:
                freq, average_profile = crop_profile_between_resolution(freq, average_profile, crop_freq[0], crop_freq[1])
                freq, y_max = crop_profile_between_resolution(freq, y_max, crop_freq[0], crop_freq[1])
                freq, y_min = crop_profile_between_resolution(freq, y_min, crop_freq[0], crop_freq[1])
            
            ax1.plot(freq**2, np.log(average_profile), 'k',alpha=1)
            ax1.fill_between(freq**2,np.log(y_max), np.log(y_min), color="grey", alpha=0.5)
            if showlegend:
                ax1.legend(["N={}".format(len(profile_list))])
        
            
            ax2.set_xticks(ax1.get_xticks())
            ax2.set_xbound(ax1.get_xbound())
            ax2.set_xticklabels([round(1/np.sqrt(x),1) for x in ax1.get_xticks()])
            
    
            ax1.set_xlabel(r'Spatial Frequency $1/d^2 [\AA^{-2}]$',fontsize=font)
            ax1.set_ylabel(r'$ln\mid F \mid $',fontsize=font)
            ax2.set_xlabel(r'$d [\AA]$',fontsize=font)
        else:
            ax1.plot(freq, average_profile, 'k',alpha=1)
            ax1.fill_between(freq,y_max, y_min,color="grey", alpha=0.5)
            
            if showlegend:
                ax1.legend(["N={}".format(len(profile_list))])
        
                
            ax2.set_xticks(ax1.get_xticks())
            ax2.set_xbound(ax1.get_xbound())
            ax2.set_xticklabels([round(1/x,1) for x in ax1.get_xticks()])
            
    
            ax1.set_xlabel(r'Spatial Frequency $1/d [\AA^{-1}]$',fontsize=font)
            ax1.set_ylabel(r'normalised $ \langle F \rangle $',fontsize=font)
            ax2.set_xlabel(r'$d [\AA]$',fontsize=font)
        
    else:
        if variation is None:
            raise UserWarning("Include a variation variable to plot radial profile with variance")
        
        if len(list_of_profiles) > 1:
            raise UserWarning("Multiple profiles given as average profile..")
            
        average_profile = list_of_profiles[0]
        
        y_max = average_profile + variation
        y_min = average_profile - variation

        fig = plt.figure()
        
        ax1 = fig.add_subplot(111)
        ax1.grid(True)
        ax2 = ax1.twiny()
        
        ax1.plot(freq**2, np.log(average_profile), 'k',alpha=1)
        ax1.fill_between(freq**2,np.log(y_max), np.log(y_min),color="grey", alpha=0.5)
        ax1.legend(["N={}".format(len(profile_list))])
        
        ax1.tick_params(axis='both',which='major')
        ax2.tick_params(axis='both',which='major')
        ax2.set_xticks(ax1.get_xticks())
        ax2.set_xbound(ax1.get_xbound())
        ax2.set_xticklabels([round(1/np.sqrt(x),1) for x in ax1.get_xticks()])
        

        ax1.set_xlabel(r'$1/d^2  [\AA^{-2}]$',fontsize=font)
        ax1.set_ylabel('$ln\mid F \mid $',fontsize=font)
        ax2.set_xlabel('$d [\AA]$',fontsize=font)

    
    if ylims is not None:
        plt.ylim(ylims)
    if yticks is not None:
        plt.yticks(yticks)
    if xlims is not None:
        plt.xlim(xlims)
    
    
    plt.tight_layout()
    return fig
def plot_emmap_section(emmap, title="EMMAP Sections"):
    import matplotlib.pyplot as plt
    
    fig = plt.figure()
    plt.title(title)
    plt.axis("off")
    zn, yn, xn = emmap.shape
    
    ax1 = fig.add_subplot(131)
    plt.title("XY plane")
    plt.imshow(emmap[zn//2,:,:])
    
    ax2 = fig.add_subplot(132)
    plt.title("YZ plane")
    plt.imshow(emmap[:,:,xn//2])
    ax2.yaxis.set_major_locator(plt.NullLocator())

    
    ax3 = fig.add_subplot(133)
    plt.title("XZ plane")
    plt.imshow(emmap[:,yn//2,:])
    ax3.yaxis.set_major_locator(plt.NullLocator())

    
    return fig

def compute_radial_profile_from_mrcs(mrc_paths,keys=None,logScale=False, ylims=None, xlims=None, crop_freq=None):
    '''
    Given a list of mrc paths, this function will extract volume data and plots a radial profile for each map. The legend by default will be the filename of the MRC map. Max six maps will be used as inputs (limit from plot_radial_profiles function)

    Parameters
    ----------
    mrc_paths : list of strings
        ["path/to/map1.mrc", "path/to/map2.mrc",..]
    keys : list of strings, optional
        ["map A", "map B",..]. The default is None.

    Returns
    -------
    radial_profiles : dict 
        Dictionary of radial profiles for each map.
    emmaps : dict
        Dictionary of emmap volumes for each map.

    '''
    import mrcfile
    from locscale.include.emmer.ndimage.profile_tools import compute_radial_profile
    
    if keys is None:
        keys = [path.split('/')[-1] for path in mrc_paths]

    mrcs = []
    for mrc in mrc_paths:
        mrcs.append(mrcfile.open(mrc))
    
    k = 0
    emmaps = {}
    radial_profiles = {}
    freq={}
    
    for mrc in mrcs:
        emmaps[keys[k]] = mrc.data
        k += 1
    for key in keys:
        radial_profiles[key] = compute_radial_profile(emmaps[key])
        radial_profiles[key] = radial_profiles[key]/radial_profiles[key].max()
        
    k = 0
    for key in keys:
        shapes = radial_profiles[key].shape[0]
        apix = mrcs[k].voxel_size.x
        freq[key] = np.linspace(1./(float(apix)*shapes), 1./(float(apix)*2), shapes,endpoint=True)
        k += 1 
    
    
    fig=plot_radial_profile(freq[keys[0]], list(radial_profiles.values()),legends=keys, logScale=logScale, showPoints=False, ylims=ylims, crop_freq=crop_freq,  xlims=xlims)
    
    for key in keys:
        radial_profiles[key] = tuple([freq,radial_profiles[key]])
        
    return fig

def plot_pwlf_fit(emmap_path, mask_path, fsc_resolution):
    '''
    Function to plot PWLF fit for a given input map

    Parameters
    ----------
    emmap_path : TYPE
        DESCRIPTION.
    mask_path : TYPE
        DESCRIPTION.
    fsc_resolution : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    '''    
    import mrcfile
    import os
    from locscale.include.emmer.ndimage.profile_tools import compute_radial_profile, frequency_array, plot_radial_profile, estimate_bfactor_through_pwlf
    from locscale.include.emmer.pdb.pdb_tools import find_wilson_cutoff
    
    emmap_name = os.path.basename(emmap_path)
    
    emmap = mrcfile.open(emmap_path).data
    apix = mrcfile.open(emmap_path).voxel_size.tolist()[0]
    rp_emmap = compute_radial_profile(emmap)
    
    freq = frequency_array(rp_emmap, apix=apix)
    wilson_cutoff = find_wilson_cutoff(mask_path=mask_path)
    
    bfactor, amplitude_zero_freq, (piecewise_linfit, z, slopes) = estimate_bfactor_through_pwlf(freq, rp_emmap, wilson_cutoff, fsc_cutoff=fsc_resolution, return_all=True)
    
    rp_fit = np.exp(piecewise_linfit.predict(freq**2))  ## Fit was trained using log scale data
    
    fig = plot_radial_profile(freq, [rp_emmap, rp_fit], legends=[emmap_name, "PWLF prediction"])
    print("Breakpoints at: {}".format((1/np.sqrt(z)).round(2)))
    
    return fig

def plot_fsc_maps(input_map_1, input_map_2, input_mask, apix, calc_fsc=0.5,font=16, legend=None):

    import matplotlib.pyplot as plt
    from scipy.interpolate import interp1d
    from emmer.ndimage.map_utils import parse_input
    from emmer.ndimage.radial_profile.profile_tools import frequency_array
    from emmer.ndimage.fsc.calculate_fsc_curve import calculate_fsc_curve
    
    emmap_1 = parse_input(input_map_1)
    emmap_2 = parse_input(input_map_2)
    mask = parse_input(input_mask)
    
    masked_emmap_1 = mask*emmap_1
    masked_emmap_2 = mask*emmap_2
    fsc = calculate_fsc_curve(masked_emmap_1, masked_emmap_2)
    freq = frequency_array(fsc, apix=apix)
    
    

    

    fig = plt.figure()
    ax1 = fig.add_subplot(111)
    ax1.grid(True)
    ax2 = ax1.twiny()
    
    ax1.plot(freq,fsc, 'b')
    ax1.plot(freq,np.ones(len(fsc))*0.143,'k--')  
    ax1.plot(freq,np.ones(len(fsc))*0.5,'k-')  
        
    ax2.set_xticks(ax1.get_xticks())
    ax2.set_xbound(ax1.get_xbound())
    ax2.set_xticklabels([round(1/x,1) for x in ax1.get_xticks()])
    if legend is None:
        legend = ['FSC curve','FSC=0.143','FSC=0.5']
    
    ax1.legend(legend,fontsize=font)
    ax1.set_xlabel(r'$1/d [\AA^{-1}]$',fontsize=font)
    ax1.set_ylabel('FSC',fontsize=font)
    ax2.set_xlabel('$d [\AA]$',fontsize=font)
        
    ## get relation between freq and fsc
    if calc_fsc:
        f = interp1d(fsc, freq)
        fsc_value = f(calc_fsc)    
        return fig, fsc_value
    else:
        return fig


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 17 15:35:54 2021

@author: mjoosten1
"""

# test file for functions in the pdb section of emmer.

# global imports
import unittest
import gemmi
import numpy as np
import os

class test_pdb_functions(unittest.TestCase):
    def setUp(self):
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path

        os.sys.path.insert(0, self.emmer_install_path)
        
        # locate data used in tests
        data_path = "/tests/pdb_unittest/data/"
        self.pdb_path = self.emmer_install_path+data_path+"testing_model.pdb"
        self.model_with_nucleotides = self.emmer_install_path+data_path+"6NY1.pdb"
        self.charged_model = self.emmer_install_path+data_path+"charged.pdb"
        self.no_cryst1_line = self.emmer_install_path+data_path+"no_cryst1_line.pdb"
        self.pdb_gemmi = gemmi.read_structure(self.pdb_path)
        
        # locate data for verification of test results
        verify_path = "/tests/pdb_unittest/verify/"
        self.verify_emmap_path = self.emmer_install_path+verify_path+"verify_map.mrc"
        self.verify_emmap_gemmi = gemmi.read_ccp4_map(self.verify_emmap_path)
        self.verify_cif = self.emmer_install_path+verify_path+"testing_model.cif"    
        self.verify_shifted = self.emmer_install_path+verify_path+"shifted_model.pdb"
        self.verify_ca_coords = self.emmer_install_path+verify_path+"ca_coords.npy"
        self.verify_all_coords = self.emmer_install_path+verify_path+"all_coords.npy"

    def test_convert_pdb_to_map(self):
        from emmer.pdb.convert.convert_pdb_to_map import convert_pdb_to_map

        temp_map_from_model, temp_grid = convert_pdb_to_map(input_pdb=self.pdb_path,
                                  size=(48,96,48),
                                  return_grid=True)
        
        map_equal_size = temp_map_from_model.shape == self.verify_emmap_gemmi.grid.shape
        m1, m2 = temp_map_from_model.mean(), np.array(self.verify_emmap_gemmi.grid).mean()
        s1, s2 = temp_map_from_model.std(), np.array(self.verify_emmap_gemmi.grid).std()
        CCC = ((np.rot90(np.flip(temp_map_from_model,axis=2), axes=(0,2)) - m1)*(np.array(self.verify_emmap_gemmi.grid) - m2)/(s1*s2)).sum() / temp_map_from_model.size
        
        self.assertTrue(map_equal_size)
        self.assertAlmostEqual(temp_map_from_model.mean(), np.array(self.verify_emmap_gemmi.grid).mean(), delta=1e-5)
        self.assertAlmostEqual(temp_map_from_model.min(), np.array(self.verify_emmap_gemmi.grid).min(), delta=1e-5)
        self.assertAlmostEqual(temp_map_from_model.max(), np.array(self.verify_emmap_gemmi.grid).max(), delta=1e-5)
        self.assertAlmostEqual(temp_map_from_model.std(), np.array(self.verify_emmap_gemmi.grid).std(), delta=1e-5)
        self.assertAlmostEqual(CCC, 1, delta=1e-3)

    def test_convert_pdb_to_mmcif(self):
        from emmer.pdb.convert import convert_pdb_to_mmcif
        
        convert_pdb_to_mmcif.convert_pdb_to_mmcif(self.pdb_path)
        test_cif = gemmi.read_structure(self.pdb_path[:-4]+".cif")
        verify_cif = gemmi.read_structure(self.verify_cif)
        
        tM = test_cif[0]
        vM = verify_cif[0]        
        tCoM = tM.calculate_center_of_mass()
        vCoM = vM.calculate_center_of_mass()
        
        self.assertEqual(tM.count_atom_sites(), vM.count_atom_sites())
        self.assertEqual(tCoM.x, vCoM.x)
        self.assertEqual(tCoM.y, vCoM.y)
        self.assertEqual(tCoM.z, vCoM.z)
                
        # remove generated file
        os.remove(self.pdb_path[:-4]+".cif")
        
    def test_check_if_gemmi_st_is_downloadable(self):
        from emmer.pdb.pdb_utils import check_if_gemmi_st_is_downloadable

        self.assertTrue(check_if_gemmi_st_is_downloadable('3j5p'))
        self.assertTrue(check_if_gemmi_st_is_downloadable('5vy5'))
        
    def test_get_gemmi_st_from_id(self):
        from emmer.pdb.pdb_utils import get_gemmi_st_from_id
        
        tmp_gemmi = get_gemmi_st_from_id('3j5p')
        self.assertEqual(tmp_gemmi[0].count_atom_sites(), 18636)
        
        tmp_gemmi = get_gemmi_st_from_id('5vy5')
        self.assertEqual(tmp_gemmi[0].count_atom_sites(), 10472)

    def test_shift_coordinates(self):
        from emmer.pdb.pdb_utils import shift_coordinates
        
        outfile = self.emmer_install_path+"/tests/pdb_unittest/data/tmp_shifted_model.pdb"
        st=shift_coordinates(input_pdb=self.pdb_path, trans_matrix=[3,3,3])
        st.write_pdb(outfile)
        ftest = open(outfile)
        lnstest = ftest.readlines()
        ftest.close()
        fverify = open(self.verify_shifted)
        lnsverify = fverify.readlines()
        fverify.close()
        shift_correct = True
        for ln in range(len(lnstest)):
            if 'ATOM' in lnstest[ln]:
                if lnstest[ln] != lnsverify[ln]:
                    shift_correct = False
                    
        self.assertTrue(shift_correct)

        os.remove(outfile)
        
    def test_split_model_based_on_nucleotides(self):
        from emmer.pdb.pdb_utils import split_model_based_on_nucleotides
        
        model_with_nucleotides = gemmi.read_structure(self.model_with_nucleotides)
        split_model = split_model_based_on_nucleotides(model_with_nucleotides)
        
        self.assertEqual(split_model[0][0].count_atom_sites(), 940)
        self.assertEqual(split_model[1][0].count_atom_sites(), 2300)

    def test_get_bfactors(self):
        from emmer.pdb.pdb_utils import get_bfactors
        
        B = get_bfactors(input_pdb=self.pdb_path)
        self.assertEqual(len(B), 125)
        
        bfac_correct = True
        for i in B:
            if i!=30:
                bfac_correct = False

        self.assertTrue(bfac_correct)
        
    def test_add_atomic_bfactors(self):
        from emmer.pdb.pdb_utils import add_atomic_bfactors
        
        outfile = self.emmer_install_path+"/tests/pdb_unittest/data/tmp_addBfac_model.pdb"
        st=add_atomic_bfactors(input_pdb=self.pdb_path, additional_biso=100)
        st.write_pdb(outfile)
        
        fid = open(outfile)
        lns = fid.readlines()
        fid.close()
        bfac_correct = True
        for ln in lns:
            if ln[:4] == "ATOM":
                if ln[60:66]!="130.00":
                    bfac_correct = False
        
        self.assertTrue(bfac_correct)
        
        os.remove(outfile)
        
    def test_set_atomic_bfactors(self):
        from emmer.pdb.pdb_utils import set_atomic_bfactors

        outfile = self.emmer_install_path+"/tests/pdb_unittest/data/tmp_setBfac_model.pdb"
        st=set_atomic_bfactors(input_pdb=self.pdb_path, b_iso=100)
        st.write_pdb(outfile)
        fid = open(outfile)
        lns = fid.readlines()
        fid.close()
        bfac_correct = True
        for ln in lns:
            if ln[:4] == "ATOM":
                if ln[60:66]!="100.00":
                    bfac_correct = False
       
        self.assertTrue(bfac_correct)

        os.remove(outfile)
    
    def test_get_residue_ca_coordinates(self):
        from emmer.pdb.pdb_utils import get_residue_ca_coordinates
        
        ca_coords = get_residue_ca_coordinates(self.pdb_path)
        verify_coords = np.load(self.verify_ca_coords, allow_pickle=True)
        coords_are_equal = (ca_coords!=verify_coords).sum() == 0

        self.assertTrue(coords_are_equal)
        
    def test_get_coordinates(self):
        from emmer.pdb.pdb_utils import get_coordinates
        
        coords = get_coordinates(self.pdb_path)
        verify_coords = np.load(self.verify_all_coords, allow_pickle=True)
        coords_are_equal = (coords!=verify_coords).sum() == 0
         
        self.assertTrue(coords_are_equal)

    def test_remove_atomic_charges(self):
        from emmer.pdb.pdb_utils import remove_charges
        
        outfile = self.emmer_install_path+"/tests/pdb_unittest/data/tmp_nocharge_model.pdb"
        st=remove_charges(self.charged_model)
        st.write_pdb(outfile)
        uncharged_gemmi = gemmi.read_structure(outfile)
        charge_removed_correct = True
        for mdl in uncharged_gemmi:
            for chn in mdl:
                for res in chn:
                    for atm in res:
                        if atm.charge != 0: charge_removed_correct = False
                        
        self.assertTrue(charge_removed_correct)      
        
        os.remove(outfile)
        
    def test_add_cryst1_line(self):
        from emmer.pdb.pdb_tools.add_cryst1_line import add_cryst1_line

        outfile = self.emmer_install_path+"/tests/pdb_unittest/data/tmp_addcryst1_model.pdb"
        add_cryst1_line(pdb_path=self.no_cryst1_line, unitcell=(12,12,12), new_pdb_path=outfile)
        fid = open(outfile)
        lns = fid.readlines()
        fid.close()
        cryst_line_present = False
        cryst_line_correct = False
        for ln in lns:
            if "CRYST1" in ln:
                cryst_line_present = True
                if ln[9:15]=='12.000' and ln[18:24]=='12.000' and ln[27:33]=='12.000':
                    cryst_line_correct = True
                
        self.assertTrue(cryst_line_present)
        self.assertTrue(cryst_line_correct)                
        
        os.remove(outfile)
        
    def test_set_to_center_of_unit_cell(self):
        from emmer.pdb.pdb_utils import set_to_center_of_unit_cell
        
        unit_cell = gemmi.UnitCell(100,100,100,90,90,90)
        centered_gemmi = set_to_center_of_unit_cell(self.pdb_gemmi, unit_cell)
        
        CoM = centered_gemmi[0].calculate_center_of_mass()
        
        self.assertAlmostEqual(CoM.x, 50, delta=1e-5)
        self.assertAlmostEqual(CoM.y, 50, delta=1e-5)        
        self.assertAlmostEqual(CoM.z, 50, delta=1e-5)
        
        num_total_atoms = 0
        for mdl in centered_gemmi:
            num_total_atoms += mdl.count_atom_sites()

        atmidx = 0
        pos = np.zeros((num_total_atoms,3))
        for mdl in centered_gemmi:
            for chn in mdl:
                for res in chn:
                    for atm in res:
                        pos[atmidx,:] = [atm.pos.x, atm.pos.y, atm.pos.z]
                        atmidx += 1
        
        self.assertAlmostEqual(pos.max(axis=0)[0], 54, delta=1)
        self.assertAlmostEqual(pos.max(axis=0)[1], 69, delta=1)
        self.assertAlmostEqual(pos.max(axis=0)[2], 54, delta=1)
        
# =============================================================================
#     def test_get_unit_cell_estimate(self):
#         from emmer.pdb.pdb_tools import get_unit_cell_estimate
#         
#         ucell_estimate = get_unit_cell_estimate(self.pdb_gemmi, 0.65)
#         self.assertTrue(ucell_estimate.a, 1.5556)
#         
# =============================================================================
    def test_find_radius_of_gyration(self):
        from emmer.pdb.pdb_utils import find_radius_of_gyration
        
        RoG = find_radius_of_gyration(self.pdb_path)
        
        self.assertEqual(RoG, 24.539793074376302)
        
if __name__ == '__main__':
    unittest.main()




## Script to measure the volume of a cryo EM map at a given threshold using the ConvexHull algorithm

def volume_at_threshold(emmap, apix, reference_threshold):
    import numpy as np

    binarised_emmap = (emmap>=reference_threshold).astype(np.int_)
    sum_of_voxels = binarised_emmap.sum()
    volume_real_units = sum_of_voxels * apix**3

    return volume_real_units

    




# -*- coding: utf-8 -*-
"""
Created on 25 April 2022

@author: alok
"""

import unittest
import gemmi
import mrcfile
import numpy as np
import os

class test_coordinate_functions(unittest.TestCase):
    def setUp(self):
        import pickle
        import emmer
        
        emmer_module_path = os.path.dirname(emmer.__file__)
        emmer_install_path = emmer_module_path
        self.emmer_install_path = emmer_install_path
        os.sys.path.insert(0, self.emmer_install_path)
        
        self.data_folder = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','data') 
        self.emmap_path = os.path.join(self.data_folder,'emd5778_map.mrc')
        self.mask_path = os.path.join(self.data_folder,'emd5778_mask.mrc')
        self.pdb_path = os.path.join(self.data_folder,'pdb3j5p.pdb')
        self.modmap_path = os.path.join(self.data_folder,'pdb3j5p_map.mrc')
        
        verify_output_file = os.path.join(self.emmer_install_path, 'tests','ndimage_unittest','verify','ndimage_outputs.pickle') 
        with open(verify_output_file, "rb") as outfile:
            self.validate = pickle.load(outfile)
        
    def copy_files(self, file_path, tempDir):
        from subprocess import run
        run(["cp",file_path,tempDir])
        if os.path.exists(os.path.join(tempDir, os.path.basename(file_path))):               
            return os.path.join(tempDir, os.path.basename(file_path))
            
        else:
            raise UserWarning("Could not copy {} to {}".format(path,tempDir))
        
    def test_coordinate_functions(self):
        from emmer.pdb.convert.convert_polar_to_cartesian import convert_polar_to_cartesian
        from emmer.pdb.convert.convert_mrc_to_pdb_position import convert_mrc_to_pdb_position
        from emmer.pdb.convert.convert_pdb_to_mrc_position import convert_pdb_to_mrc_position
        
        from scipy.spatial import distance

        rmsd_magnitude=15
        
        for trial in range(1000):
            pos = np.random.uniform(0,500,size=3)
            r = np.random.uniform(0, rmsd_magnitude*2)
            shake_thetas = np.random.uniform(0, 2*np.pi)
            shake_phis = np.random.uniform(0, np.pi)
            shake_vector = np.column_stack((r, shake_thetas, shake_phis))
            shake_v = shake_vector[0]
            new_pos = pos+convert_polar_to_cartesian(shake_v)
            d = distance.euclidean(new_pos, pos)
            self.assertTrue(abs(d-r)<0.01)

        
        for i in range(1000):
            apix = np.random.uniform(0.1,2)
            pos = np.random.uniform(0,500,size=3)
            mrcpos = convert_pdb_to_mrc_position([pos],apix)[0]
            pdbpos = convert_mrc_to_pdb_position([mrcpos],apix)[0]
            d = distance.euclidean(pdbpos, pos)
            self.assertTrue(abs(d)<apix)
            
              
        

        
        

       
        

      


      